﻿/**
 * Created by zj on 2015/10/26 0026.
 *
 */
(function() {
    MLLayer.WebGLRenderCmd = function (renderable) {
        cc.Layer.WebGLRenderCmd.call(this, renderable);
        this._needDraw = true;
        this._mat4 = new cc.math.Matrix4();
        var fragMent =
            "precision lowp float;\n"
            + "varying vec4 v_fragmentColor; \n"
            + "varying vec2 v_texCoord; \n"
            + "void main() \n"
            + "{ \n"
            + "    gl_FragColor = v_fragmentColor * texture2D(CC_Texture0, v_texCoord); \n"
            + "    gl_FragColor.a = 1.0; \n"
            + "}";
        var program = null;
        if(!program){
            program = new cc.GLProgram();
            program.initWithVertexShaderByteArray(cc.SHADER_POSITION_TEXTURE_COLOR_VERT, fragMent);
            program.addAttribute(cc.ATTRIBUTE_NAME_POSITION, cc.VERTEX_ATTRIB_POSITION);
            program.addAttribute(cc.ATTRIBUTE_NAME_COLOR, cc.VERTEX_ATTRIB_COLOR);
            program.addAttribute(cc.ATTRIBUTE_NAME_TEX_COORD, cc.VERTEX_ATTRIB_TEX_COORDS);
            program.link();
            program.updateUniforms();
        }
        this._shaderProgram = program/*cc.shaderCache.programForKey(cc.SHADER_POSITION_TEXTURECOLOR)*/;
        cc.kmMat4Identity(this._mat4);
    };

    var proto = MLLayer.WebGLRenderCmd.prototype = Object.create(cc.Layer.WebGLRenderCmd.prototype);
    proto.constructor = MLLayer.WebGLRenderCmd;

    proto._ReCreateRenderTexture = function(){
        if (this._sprite != null){
            this._sprite.setTexture(null);
        }else{
            this._sprite = new cc.Sprite();
        }

        if (this._rtOrg != null){
            this._rtOrg.cleanup();
        }
        if (this._rt1 != null){
            this._rt1.cleanup();
        }
        if (this._rt2 != null){
            this._rt2.cleanup();
        }

        var size = this._node._textureSize;
        this._rtOrg = new cc.RenderTexture(size.width, size.height);
        this._rt1 = new cc.RenderTexture(size.width, size.height);
        this._rt2 = new cc.RenderTexture(size.width, size.height);
    };

    proto.visit = function(parentCmd){
        if (!this._node.IsRenderTexture())
            cc.Node.WebGLRenderCmd.prototype.visit.call(this, parentCmd);
        else
            cc.renderer.pushRenderCommand(this);
    };

    proto.rendering = function(ctx){
        var node = this._node;
        var flag = node._render_flag;
        // 正常渲染直接返回，layer不渲染
        if (!node.IsRenderTexture())
            return;
        if (!node._visible)
            return;

        var gl = ctx || cc._renderContext;
        // 渲染原始场景
        this.RenderToBackBuffer(null);

        if (true){
            this.RenderPostEffect(flag, gl);
        }

        // 渲染到屏幕上
        this.RenderEnd(gl);
    };

    proto.RenderPostEffect = function(flag, gl){
        var rts = [this._rt1, this._rt2, this._rtOrg];
        var len = rts.length;
        var idx = 0;
        var rt = null;
        // 老照片
        if (flag & MLLayer.LAYER_SPECIA){
            rt = rts[idx%len];
            rt.beginWithClear(0, 0, 0, 0);

            this.RenderSpecia(gl);

            this._sprite.setTexture(rt.getSprite().getTexture());
            rt.end();
            idx++;
        }

        // 置灰
        if (flag & MLLayer.LAYER_GRAY){
            rt = rts[idx%len];
            rt.beginWithClear(0, 0, 0, 0);

            this.RenderGray(gl);

            this._sprite.setTexture(rt.getSprite().getTexture());
            rt.end();
            idx++;
        }

        // 反相
        if (flag & MLLayer.LAYER_NEGATIVE){
            rt = rts[idx%len];
            rt.beginWithClear(0, 0, 0, 0);

            var negativeRender = this.RenderNegative(gl);
            if (negativeRender){
                this._sprite.setTexture(rt.getSprite().getTexture());
                idx++;
            }
            rt.end();
        }

        // 闪烁
        if (flag & MLLayer.LAYER_GLOWFILTER){
            rt = rts[idx%len];
            rt.beginWithClear(0, 0, 0, 0);

            this.RenderGlowFilter(gl);

            this._sprite.setTexture(rt.getSprite().getTexture());
            rt.end();
            idx++;
        }

        // 点光照
        if (flag & MLLayer.LAYER_POINTLIGHT){
            rt = rts[idx%len];
            rt.beginWithClear(0, 0, 0, 0);

            this.RenderPointLight(gl);

            this._sprite.setTexture(rt.getSprite().getTexture());
            rt.end();
            idx++;
        }

        // 波浪效果
        if (flag & MLLayer.LAYER_WAVE){
            rt = rts[idx%len];
            rt.beginWithClear(0, 0, 0, 0);

            this.RenderWave(gl);

            this._sprite.setTexture(rt.getSprite().getTexture());
            rt.end();
            idx++;
        }

        // 曝光效果
        if (flag & MLLayer.LAYER_BLOOM){
            var bloomRtt = [rts[idx++%len], rts[idx%len]];
            this.RenderBloom(gl, bloomRtt);
            idx++;
        }
    }

    proto.RenderToBackBuffer = function(parentCmd){
        this._rtOrg.beginWithClear(0, 0, 0, 255);
        this._needDraw = false;
        this._rtOrg._renderCmd._needDraw = false;
        cc.Node.WebGLRenderCmd.prototype.visit.call(this, parentCmd);
        this._rtOrg.end();

        // 把纹理放到sprite上
        this._sprite._renderCmd._needDraw = true;
        this._needDraw = true;

        var texture = this._rtOrg.getSprite().getTexture();
        var rect = this._rtOrg.getSprite().getTextureRect();
        if (this._sprite._texture == null){
            this._sprite.initWithTexture(texture, rect);
            this._sprite.setFlippedY(true);
            this._sprite.setAnchorPoint(0, 0);
        }else{
            this._sprite.setTexture(texture);
        }

        var node = this._node;
        var spr = this._sprite, locTexture = this._sprite._texture;
        // 如果场景不需要旋转缩放，这里就可以简化，不计算transform，直接设置到屏幕上
        if (spr.x != -node.x || spr.y != -node.y){
            spr.x = -node.x;
            spr.y = -node.y;

            spr._renderCmd.transform(this);
        }
    };

    proto.RenderSpecia = function(ctx){
        var locTexture = this._sprite.getTexture();
        var specia = this._node._oSpecia;
        var cmd = this._sprite._renderCmd;

        var program = specia.shaderProgram;

        program.use();
        program._setUniformForMVPMatrixWithMat4(this._sprite._renderCmd._stackMatrix);
        program.setUniformLocationWith1f(specia.uniformDegree, specia.degree);

        this.RenderFunc(specia.blendFunc.src, specia.blendFunc.dst, locTexture, cmd, ctx);
    };

    proto.RenderGray = function(ctx){
        var locTexture = this._sprite.getTexture();
        var gray = this._node._oGray;
        var cmd = this._sprite._renderCmd;

        var program = gray.shaderProgram;

        program.use();
        program._setUniformForMVPMatrixWithMat4(this._sprite._renderCmd._stackMatrix);

        this.RenderFunc(gray.blendFunc.src, gray.blendFunc.dst, locTexture, cmd, ctx);
    };

    proto.RenderNegative = function(ctx){
        var locTexture = this._sprite.getTexture();
        var negative = this._node._oNegative;
        var cmd = this._sprite._renderCmd;

        if (negative.bIsNegative){
            var program = negative.shaderProgram;
            program.use();
            program._setUniformForMVPMatrixWithMat4(this._sprite._renderCmd._stackMatrix);
            this.RenderFunc(negative.blendFunc.src, negative.blendFunc.dst, locTexture, cmd, ctx);
            return true;
        }
        return false;
    };

    proto.RenderPointLight = function(ctx){
        var locTexture = this._sprite.getTexture();
        var lens = this._node._oPointLight;
        var cmd = this._sprite._renderCmd;

        var program = lens.shaderProgram;

        program.use();
        program._setUniformForMVPMatrixWithMat4(this._sprite._renderCmd._stackMatrix);
        program.setUniformLocationWith1f(lens.uniformRadius, lens.fRadius);
        program.setUniformLocationWith1f(lens.uniformSpot, lens.fIsSpot);
        program.setUniformLocationWith2f(lens.uniformLightPos, lens.lightPos.x, lens.lightPos.y);
        program.setUniformLocationWith2f(lens.uniformTextureSize, lens.textureSize.x, lens.textureSize.y);
        this.RenderFunc(lens.blendFunc.src, lens.blendFunc.dst, locTexture, cmd, ctx);
    };

    proto.RenderGlowFilter = function(ctx){
        var locTexture = this._sprite.getTexture();
        var glow = this._node._oGlowFilter;
        var cmd = this._sprite._renderCmd;

        var program = glow.shaderProgram;

        program.use();
        program._setUniformForMVPMatrixWithMat4(this._sprite._renderCmd._stackMatrix);
        program.setUniformLocationWith1f(glow.uniformTime, glow.fTime);
        this.RenderFunc(glow.blendFunc.src, glow.blendFunc.dst, locTexture, cmd, ctx);
    };

    proto.RenderWave = function(ctx){

        var locTexture = this._sprite.getTexture();
        var wave = this._node._oWaveFull;
        var cmd = this._sprite._renderCmd;

        var program = wave.shaderProgram;

        //this._mat4.mat[14] = this._stackMatrix.mat[14];
        program.use();
        program._setUniformForMVPMatrixWithMat4(this._sprite._renderCmd._stackMatrix);
        program.setUniformLocationWith2f(wave.uniformResolution, wave.resolution.x, wave.resolution.y);
        program.setUniformLocationWith1f(wave.uniformTime, wave.fTime);

        this.RenderFunc(wave.blendFunc.src, wave.blendFunc.dst, locTexture, cmd, ctx);
    };

    proto.RenderBloom = function(ctx, rts){
        var rt1 = rts[0];
        var rt2 = rts[1]

        var gl = ctx;

        rt1.beginWithClear(0, 0, 0, 0);
        var orgTexture = this._sprite.getTexture();
        // 先提取亮色
        var locTexture = orgTexture;
        var bloom = this._node._oBloom;
        var cmd = this._sprite._renderCmd;
        var param = bloom.TakeParam();

        var program = bloom.highShaderProgram;
        program.use();
        program._setUniformForMVPMatrixWithMat4(this._sprite._renderCmd._stackMatrix);
        program.setUniformLocationWith1f(bloom.highShaderUniforThreshold, param[0]);
        this.RenderFunc(bloom.blendFunc.src, bloom.blendFunc.dst, locTexture, cmd, ctx);
        rt1.end();

        // 叠加原图
        rt2.beginWithClear(0, 0, 0, 0);
        var blurTexture = rt1.getSprite().getTexture();
        program = bloom.bloomShaderProgram;
        program.use();
        program._setUniformForMVPMatrixWithMat4(this._sprite._renderCmd._stackMatrix);
        program.setUniformLocationWith1f(bloom.bloomShaderUniformBloomIntensity, param[1]);
        program.setUniformLocationWith1f(bloom.bloomShaderUniformBaseIntensity, param[2]);
        program.setUniformLocationWith1f(bloom.bloomShaderUniformBloomSaturation, param[3]);
        program.setUniformLocationWith1f(bloom.bloomShaderUniformBaseSaturation, param[4]);

        var tex1Loc = program.getUniformLocationForName("CC_Texture0");
        var tex2Loc = program.getUniformLocationForName("CC_BlurTex");
        gl.activeTexture(gl.TEXTURE0);
        gl.bindTexture(gl.TEXTURE_2D, orgTexture.getName());
        gl.uniform1i(tex1Loc, 0);
        gl.activeTexture(gl.TEXTURE1);
        gl.bindTexture(gl.TEXTURE_2D, blurTexture.getName());
        gl.uniform1i(tex2Loc, 1);

        cc.glBlendFunc(bloom.blendFunc.src, bloom.blendFunc.dst);

        cc.glEnableVertexAttribs(cc.VERTEX_ATTRIB_FLAG_POS_COLOR_TEX);
        gl.bindBuffer(gl.ARRAY_BUFFER, cmd._quadWebBuffer);
        gl.vertexAttribPointer(0, 3, gl.FLOAT, false, 24, 0);                   //cc.VERTEX_ATTRIB_POSITION
        gl.vertexAttribPointer(1, 4, gl.UNSIGNED_BYTE, true, 24, 12);           //cc.VERTEX_ATTRIB_COLOR
        gl.vertexAttribPointer(2, 2, gl.FLOAT, false, 24, 16);                  //cc.VERTEX_ATTRIB_TEX_COORDS
        gl.drawArrays(gl.TRIANGLE_STRIP, 0, 4);

        gl.activeTexture(gl.TEXTURE0);
        gl.bindTexture(gl.TEXTURE_2D, null);
        gl.activeTexture(gl.TEXTURE1);
        gl.bindTexture(gl.TEXTURE_2D, null);        //使用完必须置为空，否则影响其他node
        rt2.end();
        cc.g_NumberOfDraws++;

        this._sprite.setTexture(rt2.getSprite().getTexture());
    };

    proto.RenderFunc = function(src, dst, tex, cmd, gl){
        cc.glBlendFunc(src, dst);
        //optimize performance for javascript
        cc.glBindTexture2DN(0, tex);
        cc.glEnableVertexAttribs(cc.VERTEX_ATTRIB_FLAG_POS_COLOR_TEX);

        gl.bindBuffer(gl.ARRAY_BUFFER, cmd._quadWebBuffer);
        if (cmd._quadDirty) {
            gl.bufferData(gl.ARRAY_BUFFER, cmd._quad.arrayBuffer, gl.DYNAMIC_DRAW);
            cmd._quadDirty = false;
        }
        gl.vertexAttribPointer(0, 3, gl.FLOAT, false, 24, 0);                   //cc.VERTEX_ATTRIB_POSITION
        gl.vertexAttribPointer(1, 4, gl.UNSIGNED_BYTE, true, 24, 12);           //cc.VERTEX_ATTRIB_COLOR
        gl.vertexAttribPointer(2, 2, gl.FLOAT, false, 24, 16);                  //cc.VERTEX_ATTRIB_TEX_COORDS
        gl.drawArrays(gl.TRIANGLE_STRIP, 0, 4);

        cc.g_NumberOfDraws++;
    };

    proto.RenderEnd = function(ctx){
        var cmd = this._sprite._renderCmd;
        var program = this._shaderProgram;
        var locTexture = this._sprite._texture;
        program.use();
        program._setUniformForMVPMatrixWithMat4(cmd._stackMatrix);

        this.RenderFunc(this._sprite._blendFunc.src, this._sprite._blendFunc.dst, locTexture, cmd, ctx);
    };
})();