﻿/**
 * 资源加载单元
 * pbk 2015年10月16日
 */
var LoadCell = cc.Class.extend({
	ctor:function(){
		var self = this;
		
		self.sUrl = null;       // 资源加载地址
		self.oTarget = null;    // 加载对象
		self.iPriority = null;  // 资源加载优先级
		self.iType = null;      // 资源类型
		self.args = null;       // 加载完成回调参数
		self.bInPool = false;   // 是否在对象池中
	},

	/**
	 * 清理加载单元
	 */
	Clean:function(){
		var self = this;
		
		self.sUrl = null;
		self.oTarget = null;
		self.iPriority = null;
		self.iType = null;
		self.args = null;
	},

	/**
	 * 销毁加载单元
	 */
	Dispose:function(){
		this.Clean();
		
		this.bInPool = null;
	},

	/**
	 * 加载完成回调函数
	 * @param 加载返回参数
	 */
	Complete:function( data ){
		var obj = this.args;
		
		if ( obj == null )
			return;
		
		if ( obj.complete == null || typeof(obj.complete) != "function" )
			return;
		
		if ( obj.completeParam == null ){
			obj.complete.call( this.oTarget, data );
		} else {
			obj.complete.call( this.oTarget, data, obj.completeParam );
		}
	},
	
	/**
	 * 加载错误回调函数
	 */
	Error:function(){
		var obj = this.args;

		if ( obj == null )
			return;

		if ( obj.error == null || typeof(obj.error) != "function" )
			return;

		if ( obj.errorParam == null ){
			obj.error.call( this.oTarget );
		} else {
			obj.error.call( this.oTarget, obj.errorParam );
		}
	},
	
	/**
	 * 加载进度回调函数
	 * @param data 单个加载返回数据
	 * @param index 数据对应的下标
	 * @param total 总加载个数
	 * @param cur 当前加载完成个数
	 */
	Progress:function( data, index, total, cur ){
		var obj = this.args;
		
		if ( obj == null )
			return;
		
		if ( obj.progress == null || typeof(obj.progress) != "function" )
			return;
		
		if ( obj.progressParam == null ){
			obj.progress.call( this.oTarget, data, index, total ,cur );
		} else {
			obj.progress.call( this.oTarget, data, index, total ,cur, obj.progressParam );
		}
	},
	
	/**
	 * 加载对象是否被销毁
	 */
	TargetIsDispose:function(){
		var tar = this.oTarget;
		
		if ( tar == null )
			return false;
		
		if ( tar.hasOwnProperty("bDispose") )
			return tar.bDispose === true;
			
		return false;
	},
	
	/**
	 * 是否缓存
	 */
	IsCache:function(){
		var obj = this.args;

		if ( obj == null )
			return true;
		
		return obj.hasOwnProperty("cache") ? obj.cache === true : true;
	}
});