﻿/**
 * Created by zj on 2015/10/19 0019.
 */

var ParticleEmitter = cc.Class.extend({
    ctor:function(xmlDoc, tex){
        this._moveProxy = null;
        this._emitProxy = null;
        this._colorProxy = null;
        this._texAtlas = null;
        this._iNumParticles = 0;
        this._fFrameTime = 0;
        this._fRotateZAngle = 0;
        this._fCompleteCallBack = null;
        this._render = new ParticleEmitterRender(this);
        this._config = new ParticleProperty();
        this._config.ParseConfig(xmlDoc);
        this._strName = this._config.emitterName;

        this._texture = tex;
        this._listParticles = [];
        this._fEmitLife = 0;
        this._fEmitRate = 0;
        this._fEmitX = this._fEmitY = 0;
        this._bTurnX = this._config.particleTurnX;
        this._bTurnY = this._config.particleTurnY;
        this._bSingleParticle = this._config.particleSingle;
        this._blendSrc = this._config.emitterBlendFuncSource;
        this._blendDst = this._config.emitterBlendFuncDestination;
        this._particleRegion = new cc.Rect(0, 0, this._texture.width, this._texture.height);
        this._configParticleAngle = this._config.particleAngle.LifeToVariance(0);
        this._isRotateX = this._config.emitterRotateXAxis;

        this._PreLoadFrame(this._config.emitterPreFrame);
        this._CreateColorProxy();
        this._CreateEmitProxy();
        this._CreateMoveProxy();
        if (ParseUtil.HasTextureAtlas(xmlDoc)){
            this._texAtlas = new StarlingAtlas(xmlDoc);
        }

        this._render.RaiseCapacity(128);
        this._render.CreateFilter();
    },

    _CreateMoveProxy:function(){
        if (this._config.emitterMoveProxy == EnumMoveProxy.E_GRAVITY){
            this._moveProxy = new GravityProxy();
        }else{
            this._moveProxy = new RadialProxy();
        }
    },

    _CreateEmitProxy:function(){
        if (this._config.emitterEmitProxy == EnumEmitProxy.E_LINE){
            this._emitProxy = new EmitterLineProxy();
        }else if (this._config.emitterEmitProxy == EnumEmitProxy.E_CIRCLE){
            this._emitProxy = new EmitterCircleProxy();
        }else if (this._config.emitterEmitProxy == EnumEmitProxy.E_AREA){
            this._emitProxy = new EmitterAreaProxy();
        }else{
            this._emitProxy = new EmitterPointProxy();
        }
        this._emitProxy.SetInitData(this._config);
    },

    _CreateColorProxy:function(){
        if (this._colorProxy == null){
            this._colorProxy = new ColorProxy();
        }
        this._colorProxy.ParseConfig(this._config.particleColor, this._config.particleAlpha);
    },

    _PreLoadFrame:function(frameCount){
        var perFrameTime = 60 / 1000;
        this._fFrameTime = perFrameTime * frameCount;
    },

    SetCompleteCallBack:function(func){
        this._fCompleteCallBack = func;
    },

    CreateParticle:function(){
        return new ParticleExt();
    },

    InitAParticle:function(particle){
        var config = this._config;
        if (this._texAtlas != null){
            particle.ParseAtlas(this._texAtlas, config.particleTextureAtlasFPS, config.particleTextureAtlasStartFrame);
        }

        var life = this.GetLife();
        var lifespan = config.emitterParticleLifeSpan.ToValue(life) +  this._Random(config.emitterParticleLifeSpanVariance.ToValue(life));

        particle.currentTime = 0.0;
        particle.totalTime = lifespan > 0.0 ? lifespan : 0.0;

        if (lifespan <= 0.0){
            return;
        }

        particle.emitRotateZAngle = this._fRotateZAngle;

        this._emitProxy.InitParticle(particle, life);
        particle.emitterAngle = this._CalMoveRotaion(0, 0, particle.emitterOffX, particle.emitterOffY);

        // pos
        particle.startX = this._fEmitX + particle.emitterOffX;
        particle.startY = this._fEmitY + particle.emitterOffY;
        particle.x = particle.startX + config.emitterXOffset.ToValue(life) + this._Random(config.emitterXOffsetVariance.ToValue(life));
        particle.y = particle.startY + config.emitterYOffset.ToValue(life) + this._Random(config.emitterYOffsetVariance.ToValue(life));
        particle.emitX = this._fEmitX;
        particle.emitY = this._fEmitY;

        var texuteW = this._texture.width;
        var texuteH = this._texture.height;
        if (particle.IsTextureAtlas())
        {
            texuteW = particle.frames[particle.frameNum].width;
            texuteH = particle.frames[particle.frameNum].height;
        }

        // size
        var startWidth = config.emitterWidth.ToValue(life) + this._Random(config.emitterWidthVariance.ToValue(life));
        if (startWidth < 0.1) startWidth = 0.1;
        var startHeight = 0;
        if (config.particleHeightWidth)
            startHeight =startWidth;
        else
            startHeight = config.emitterHeight.ToValue(life) + this._Random(config.emitterHeightVariance.ToValue(life));

        if (startHeight < 0.1) {
            startHeight = 0.1;
        }
        particle.scaleX = startWidth / texuteW;
        particle.scaleY = startHeight / texuteH;
        particle.startWidht = startWidth;
        particle.startHeight = startHeight;

        // visibily
        particle.visibility = config.emitterVisibility.ToValue(life);

        // rotation
        var startRotation = 0.0;

        // emit angle
        var angle = config.emitterAngle.ToValue(life) + this._Random(config.emitterAngleVariance.ToValue(life));
        angle += particle.emitterAngle;

        if (this._configParticleAngle.fLife == 0)// �̶��Ƕ�
            startRotation = this._configParticleAngle.x;
        else if (this._configParticleAngle.fLife == 1)// ����Ƕ�
            startRotation = this._configParticleAngle.y + Math.random() * this._configParticleAngle.x;
        else if (this._configParticleAngle.fLife == 2)// �˶�����
            startRotation = angle + this._configParticleAngle.x;
        else if (this._configParticleAngle.fLife == 3)// ���뵽�˶���
            startRotation = angle + this._configParticleAngle.x;

        var speed = config.emitterSpeed.ToValue(life) + this._Random(config.emitterSpeedVariance.ToValue(life));
        particle.velocityX = speed * Math.cos(angle);
        particle.velocityY = speed * Math.sin(angle);

        particle.rotation = startRotation;

        // ��ת�ٶ�Ҫ�������õ�10��
        particle.rotationSpeed = config.emitterRotation.ToValue(life) * 10 + this._Random(config.emitterRotationVariance.ToValue(life)) * 10;

        // gravity
        particle.gravityX = config.emitterGravityX.ToValue(life) + this._Random(config.emitterGravityXVariance.ToValue(life));
        particle.gravityY = config.emitterGravityY.ToValue(life) + this._Random(config.emitterGravityYVariance.ToValue(life));

        // radial acc
        particle.radialAcceleration = config.emitterRadialAcceleration.ToValue(life) + this._Random(config.emitterRadialAccelerationVariance.ToValue(life));
        particle.tangentialAcceleration = config.emitterTangentialAcceleration.ToValue(life) + this._Random(config.emitterTangentialAccelerationVariance.ToValue(life));

        // radius
        var startRadius = config.emitterMaxRadius.ToValue(life) + this._Random(config.emitterMaxRadiusVariance.ToValue(life));
        var endRadius   = config.emitterMinRadius.ToValue(life) + this._Random(config.emitterMinRadiusVariance.ToValue(life));
        particle.emitRadius = startRadius;
        particle.emitRadiusDelta = (endRadius - startRadius) / particle.totalTime;

        // ��ת�˶�
        particle.emitRotation = startRotation;
        particle.emitRotationDelta = config.emitterRotatePerSecond.ToValue(life) + this._Random(config.emitterRotatePerSecondVariance.ToValue(life));
    },

    UpdateAParticle:function(particle, passedTime){
        var lifePer = particle.LifePer();
        var restTime = particle.totalTime - particle.currentTime;
        passedTime = restTime > passedTime ? passedTime : restTime;
        particle.currentTime += passedTime;
        // textureAtlas
        particle.AdvanceTime(passedTime);

        var config = this._config;
        // size
        var scaleXPer = config.particleWidthPer.ToValue(lifePer);
        var scaleYPer = 0.0;
        if (config.particleHeightWidth)
            scaleYPer = scaleXPer;
        else
            scaleYPer = config.particleHeightPer.ToValue(lifePer);

        var texuteW = this._texture.width;
        var texuteH = this._texture.height;
        if (particle.IsTextureAtlas())
        {
            texuteW = particle.frames[particle.frameNum].width;
            texuteH = particle.frames[particle.frameNum].height;
        }

        particle.scaleX = particle.startWidht *  scaleXPer / texuteW;
        if (particle.scaleX < 0.1)
            particle.scaleX = 0.1;
        particle.scaleY = particle.startHeight *  scaleYPer / texuteH;
        if (particle.scaleY < 0.1)
            particle.scaleY = 0.1;

        // color
        this._colorProxy.DoParticleColor(particle, lifePer);
        particle.color = particle.colorArgb.ToRgb();
        particle.alpha = particle.colorArgb.alpha * 255;

        // position
        var lastX = particle.x;
        var lastY = particle.y;
        this._moveProxy.DoMove(particle, passedTime, this);

        // rotation
        var rotationConfig = this._configParticleAngle;
        if (rotationConfig.fLife == 3)
        {// ���뵽�˶���
            particle.rotation = this._CalMoveRotaion(lastX, lastY, particle.x, particle.y) + rotationConfig.x;
        }
        else
        {// ��ת
            particle.rotation += config.particleRotationPer.ToValue(lifePer) * particle.rotationSpeed * passedTime;
        }
    },

    /**
    * ÿ֡����
    * */
    OnFrame:function (dt){
        var life = this.GetLife();
        this._fEmitRate = this._config.emitterEmitRate.ToValue(life) + this._Random(this._config.emitterEmitRateVariance.ToValue(life));
        this._fEmitRate = this._fEmitRate < 0 ? 0 : this._fEmitRate;

        this._fRotateZAngle += this._config.emitterRotateZAxisSpeed * dt * 10;
        this._render._filter.OnFrame();
        this._UpdateAllParticles(dt);
    },

    _UpdateAllParticles:function(passedTime){
        var particleIndex = 0;
        var particle;

        // �����Ѵ�����
        while (particleIndex < this._iNumParticles){
            particle = this._listParticles[particleIndex];

            if (particle.currentTime < particle.totalTime)
            {
                this.UpdateAParticle(particle, passedTime);
                    this._render.UpdateQuadWithParticle(particle, particleIndex);
                ++particleIndex;
            }
            else
            {
                if (particleIndex != this._iNumParticles - 1)
                {
                    var nextParticle = this._listParticles[parseInt(this._iNumParticles-1)];
                    this._listParticles[parseInt(this._iNumParticles-1)] = particle;
                    this._listParticles[particleIndex] = nextParticle;
                }

                --this._iNumParticles;
            }
        }
        // ���������
        if (this._fEmitLife > 0 && this._fEmitRate != 0)
        {
            var timeBetweenParticles = 1.0 / this._fEmitRate;
            this._fFrameTime += passedTime;

            var loopTimes = this._emitProxy.GetNumPoint();
            var maxNum = this._bSingleParticle ? 1 : Number.MAX_VALUE;

            var particles = this._listParticles;
            var listLen = this._render._quads.length;
            while (this._fFrameTime > 0)
            {
                if (this._iNumParticles < maxNum)
                {
                    for (var i = 0; i < loopTimes; ++i){

                        if (this._iNumParticles == listLen){
                            listLen = listLen << 1;
                            this._render.RaiseCapacity(listLen);
                        }

                        particle = particles[this._iNumParticles];
                        particle.pointIndex= i;

                        this._colorProxy.DoParticleColor(particle, 0);

                        this.InitAParticle(particle);

                        // particle might be dead at birth
                        if (particle.totalTime > 0.0)
                        {
                            this.UpdateAParticle(particle, this._fFrameTime);
                                this._render.UpdateQuadWithParticle(particle, this._iNumParticles);
                            ++this._iNumParticles;
                        }

                        if (i == loopTimes - 1){
                            this._colorProxy.DecCount();
                        }
                    }
                }

                this._fFrameTime -= timeBetweenParticles;
            }
        }

        //
        if (this._fEmitLife != Number.MAX_VALUE){
            this._fEmitLife = Math.max(0.0, this._fEmitLife - passedTime);
        }
        if (this._iNumParticles == 0 && this._fEmitLife == 0){
            // �����ص���
            this._fCompleteCallBack(this);
        }

        this._render.PostStep();
    },

    Start:function(){
        this._fEmitLife = this._config.emitterLifeSpan;
        if (this._fEmitLife == 0){
            this._fEmitLife = Number.MAX_VALUE;
        }
    },

    Stop:function(){
        this._fEmitLife = 0.0;
        this._iNumParticles = 0;
    },

    Dispose:function(){
        this.Stop();
        this._render.Dispose();
        this._render = null;
        this._listParticles.length = 0;
        this._listParticles = null;
        this._moveProxy = null;
        this._emitProxy = null;
        this._colorProxy = null;
        this._texAtlas = null;
        this._fCompleteCallBack = null;
        this._texture = null;
        this._configParticleAngle = null;
    },

    _CalMoveRotaion:function(startX, startY, nowX, nowY)
    {
        var ret = 0;
        var offx = nowX - startX;
        var offy = nowY - startY;
        // ��3��4���޷���ֵΪ������ת������ת������
        if (offx < 0 && offy < 0)
        {// ��������
            ret = Math.PI + Math.atan2(-offy, -offx);
        }
        else if (offx > 0 && offy < 0)
        {// ��������
            ret = 2*Math.PI - Math.atan2(-offy, offx);
        }
        else
        {
            ret = Math.atan2(offy, offx);
        }
        return ret;
    },

    _Random:function(value){
        return value * (Math.random() * 2.0 - 1.0);
    },

    GetRender:function(){ return this._render;},

    GetIsRotateX:function(){ return this._isRotateX;},

    GetParticleRegion:function(){ return this._particleRegion;},

    GetParticlesList:function(){ return this._listParticles;},

    GetIsEmitting:function() { return this._fEmitLife > 0; },
    GetNumParticles:function() { return this._iNumParticles; },

    GetEmitterX:function() { return this._fEmitX; },
    SetEmitterX:function(value) { this._fEmitX = value },

    GetEmitterY:function() { return this._fEmitY; },
    SetEmitterY:function(value) { this._fEmitY = value },

    GetBlendFactorSource:function() { return this._blendSrc; },
    SetBlendFactorSource:function(value) { this._blendSrc = value; },

    GetBlendFactorDestination:function() { return this._blendDst; },
    SetBlendFactorDestination:function(value) { this._blendDst = value; },

    GetTexture:function(){ return this._texture;},
    SetTexture:function(value) {
        this._texture = value;
        this._particleRegion.width = value.width;
        this._particleRegion.height = value.height;
    },

    GetTurnX:function(){ return this._bTurnX;},
    SetTurnX:function(value){ this._bTurnX = value;},

    GetTurnY:function(){ return this._bTurnY;},
    SetTurnY:function(value){ this._bTurnY = value;},

    GetLife:function(){ return this._config.emitterLifeSpan -  this._fEmitLife},

    GetEmitRate:function(){ return this._fEmitRate;},
    SetEmitRate:function(value){
        this._fEmitRate = (value < 0 ? 0 : value);
    },

    GetEmitRotateZAngle:function(){  return this._fRotateZAngle; },

    GetEmitterName:function(){ return this._strName;},

    GetConfig:function(){ return this._config;}
});

var _p = ParticleEmitter.prototype;
cc.defineGetterSetter(_p, "isEmitting", _p.GetIsEmitting, null);
cc.defineGetterSetter(_p, "numParticles", _p.GetNumParticles, null);
cc.defineGetterSetter(_p, "emitterX", _p.GetEmitterX, _p.SetEmitterX);
cc.defineGetterSetter(_p, "emitterY", _p.GetEmitterY, _p.SetEmitterY);
cc.defineGetterSetter(_p, "blendFactorSource", _p.GetBlendFactorSource, _p.SetBlendFactorSource);
cc.defineGetterSetter(_p, "blendFactorDestination", _p.GetBlendFactorDestination, _p.SetBlendFactorDestination);
cc.defineGetterSetter(_p, "texture", _p.GetTexture, _p.SetTexture);
cc.defineGetterSetter(_p, "turnX", _p.GetTurnX, _p.SetTurnX);
cc.defineGetterSetter(_p, "turnY", _p.GetTurnY, _p.SetTurnY);
cc.defineGetterSetter(_p, "life", _p.GetLife, null);
cc.defineGetterSetter(_p, "emitRate", _p.GetEmitRate, _p.SetEmitRate);
cc.defineGetterSetter(_p, "emitRotateZAngle", _p.GetEmitRotateZAngle, null);
cc.defineGetterSetter(_p, "name", _p.GetEmitterName, null);
cc.defineGetterSetter(_p, "config", _p.GetConfig, null);