﻿/**
 * 地图格子类 （单例）
 * 把地图虚拟的以格子划分，保存格子阻挡等信息，方便管理对象和寻路等,
 * yq
 * tyevlag
 */

var EnumSceneGridType = EnumSceneGridType || {
	// 空格子 不能到达点
	BARRIER:0x00000000,	
	// 可行走或不可走
	W_UF:0x00000001,
	// 可跳不可跳
	UW_F:0x00000002,
	// 可行走，可飞越
	EMPTY:0x00000003,
	// 遮盖(可以和前面组合)
	COVERING:0x00000004,  
	// 安全区
	SAFE:0x00000008,
	// 不可站立(可以和前面组合)
	USTOP:0x00000010,
	// 深水
	WATER:0x00000020,  
	// 浅水
	WATERLESS:0x00000040,
	// 传送点
	TRANS:0x00000080,
	// 最大值，插值记得调整（ 类型最大值为FF 高 8位存 陷阱点编号）
	MaxType:0x0000ffff,

	// 判断用
	WORK:0x1,
	FLY:0x2,
	COV:0x4,
};

var MMapGrid = MMapGrid || {
		// 不需要阻挡
		NO_BLOCK:false,
		
		DIROFFSET:[[0, -1],
		           [1, -1],
		           [1, 0],
		           [1, 1],
		           [0, 1],
		           [-1, 1],
		           [-1, 0],
		           [-1, -1],
		           [0, -1],
		           [1, -1],
		           [1, 0],
		           [1, 1],
		           [0, 1],
		           [-1, 1],
		           [-1, 0],
		           [-1, -1]],

		nWidth:0,
		nHeight:0,
		nCurMapResId:0,

		bDebugMode:false,
		nGridWMax:0,
		nGridHMax:0,

		// 地图静态阻挡数据 二维
		aGrids:null,
		// 地图动态阻挡数据 二维
		aGridsDynamic:null,
		// 地图额外阻挡数据 二维
		aGridsExtra:null,
		// 地图格子对象数据，二维，数组列表
		aObjListGrids:null,

		// 寻路连通节点数据 二维
		aConnectType:null,

		// 寻路
		oPathFinderf:null,
		oAstarGrid:null,

		// 是否标记对象阻挡
		bMarkNpcBolck:true,
		bMarkMonsterBolck:true,
		bMarkPlayerBolck:true,
		bMarkPetBolck:true,
		bMarkCheckDead:true,

		aSceneEffectData:null,
		oDebugPath:null,
		oEndpt:null,
		
		Init:function(){
			this.oEndpt = cc.p(-1, -1);
			return this;
		},

// void Update()
// {
// if (Input.GetKeyDown(KeyCode.End))
// {
// bDebugMode = !bDebugMode;
// }
//
// if (Input.GetKeyDown(KeyCode.Home))
// {
// Vector2 start = new Vector2();// Mgr.mainPlayer.gridPt;
// Vector2 end = new Vector2(UnityEngine.Random.Range(0, nGridWMax - 1),
// UnityEngine.Random.Range(0, nGridHMax - 1));
// DebugTool.Log(" KeyCode.Home To : " + end.ToString());
// FindPaths(start, end);
// }
//
// if (Input.GetKeyDown(KeyCode.PageDown) && oEndpt.x != -1 && oEndpt.y != -1)
// {
// Vector2 start = new Vector2();//Mgr.mainPlayer.gridPt;
// FindPaths(start, oEndpt);
// }
//
// drawGrid();
// drawDebugPath();
// }
		
		LoadMap:function(id, w, h, bdata){
			this.nCurMapResId = id;
			this.nWidth = w;
			this.nHeight = h;
			
			var stream = new BinStream(bdata);
			var version = stream.ReadInt(); 
			var mapId = stream.ReadInt(); 

			var blockLen = stream.ReadInt();
			var blockDatas = stream.ReadBytes(blockLen);

			var alphaLen = stream.ReadInt();
			var alphaDatas = stream.ReadBytes(alphaLen);

			var safeLen = stream.ReadInt();
			var safeDatas = stream.ReadBytes(safeLen);

			var jumpLen = stream.ReadInt();
			var jumpDatas = stream.ReadBytes(jumpLen);

			var effLen = stream.ReadInt();
			var effDatas = stream.ReadBytes(effLen);

			var spData = null;
			var effectData = null;
			var soundData = null;
			var disData = null;
			
			if ( version == 20141228 ){
				var dataLen = null;
				var flag = null;
				
				while ( stream.bytesAvailable() ){
					dataLen = stream.ReadInt();
					flag = dataLen >> 24;
					dataLen = dataLen & 0xffffff;
					if ( flag == 0x1 ){// 特殊区域
						spData = stream.ReadBytes( dataLen );
					} else if ( flag == 0x2 ) {// 特效区域
						effectData = stream.ReadBytes( dataLen );
					} else if ( flag == 0x3 ) {// 音效区
						zz.Log( "此地图有音效：" + mapId );
						soundData = stream.ReadBytes( dataLen );
					} else if ( flag == 0x4 ) {// 远景区
						disData = stream.ReadBytes( dataLen );
					}
				}
			} else {
				if ( stream.bytesAvailable() ){
					var dataLen = stream.ReadInt();
					if ( dataLen != Number.MAX_VALUE ){
						spData = stream.ReadBytes( dataLen * 4 );
					}
					
					if ( stream.bytesAvailable() ){
						dataLen = stream.ReadInt();
						effectData = stream.ReadBytes( stream.length() - stream._pos );
					}
				}
			}
			
			this.nGridHMax = parseInt(this.nHeight / MGameDefine.GRID_HEIGHT) + (((this.nHeight % MGameDefine.GRID_HEIGHT) != 0) ? 1 : 0);
			this.nGridWMax = parseInt(this.nWidth / MGameDefine.GRID_WIDTH) + (((this.nWidth % MGameDefine.GRID_WIDTH) != 0) ? 1 : 0);

			this.oAstarGrid = new Grid(this.nGridWMax, this.nGridHMax);
			
//			if (br.PeekChar() != -1)
//			{
//				int dataLen = ToolStruct.ReadInt32(br.ReadBytes(4));
//				if (dataLen != int.MaxValue && false)
//				{
//					aGridsExtra = new int[nGridWMax, nGridHMax];
//					byte[] d = br.ReadBytes(dataLen * 4);
//					for (int i = 0; i < dataLen; ++i)
//					{
//						int tx = i % nGridWMax;
//						int ty = i / nGridWMax;
//						aGridsExtra[tx, ty] = ToolStruct.ReadInt32(BitConverter.GetBytes(BitConverter.ToInt32(d, i * 4)));
//					}
//				}
//				if (br.PeekChar() != -1 && false)
//				{
//					try
//					{
//						dataLen = 0;
//						if (s.Position <= s.Length - 4)
//						{
//							dataLen = br.ReadInt32();
//						}
//						if (s.Position <= s.Length - dataLen)
//						{
//							aSceneEffectData = br.ReadBytes(dataLen);
//						}                    
//					}
//					catch
//					{
//
//					}
//				}
//			}

			this.aGrids = ToolBase.CreateTwoArray(this.nGridWMax, this.nGridHMax, 0);
			this.aGridsDynamic = ToolBase.CreateTwoArray(this.nGridWMax, this.nGridHMax, 0);
			this.aConnectType = ToolBase.CreateTwoArray(this.nGridWMax, this.nGridHMax, 0);
			this.aObjListGrids = ToolBase.CreateTwoArray(this.nGridWMax, this.nGridHMax, null);

			// 数据填充
			// 这里一个byte描述了8个格子，1个bit一个
			// int blockIndex = 0;
			// int dataLength = effDatas.Length;
			var dataLength = this.nGridWMax * this.nGridHMax;

			var bolckDatasStream = new BinStream(blockDatas);
			var alphaDatasStream = new BinStream(alphaDatas);
			var jumpDatasStream = new BinStream(jumpDatas);
			var safeDatasStream = new BinStream(safeDatas);
			var effDatasStream = new BinStream(effDatas);
			
			var time = TimerUtil.GetTimer();
			for (var i = 0; i < dataLength; ++i){
				var index = Math.floor(i / 8);
				var offset = i % 8;

				bolckDatasStream._pos = index;
				alphaDatasStream._pos = index;
				jumpDatasStream._pos = index;
				safeDatasStream._pos = index;
				effDatasStream._pos = i;
				
				// 这些是位
				var tmpBlock = bolckDatasStream.ReadByte();
				var tmpAlpha = alphaDatasStream.ReadByte();
				var tmpJump = jumpDatasStream.ReadByte();
				var tmpSafe = safeDatasStream.ReadByte();
				// 这个是byte
				var tmpeff = effDatasStream.ReadByte();
				

				var block = ((tmpBlock >> (7 - offset)) & 0x1);
				if (this.NO_BLOCK)
					block = 1;
				
				var alpha = ((tmpAlpha >> (7 - offset)) & 0x1);
				var jump = ((tmpJump >> (7 - offset)) & 0x1);
				var safe = ((tmpSafe >> (7 - offset)) & 0x1);

				var water = (tmpeff & 0x1);
				var watersmall = ((tmpeff >> 1) & 0x1);

				// 这个就不是位了，是唯一的数
				var extraFlag = (tmpeff >> 2);
				// 开启全地图默认可跳
				var type = ((watersmall << 6) | (water << 5) | (safe << 3) | (alpha << 2) | (0x1 << 1) | (block & 0x1)) & 0xff;

				var tx = i % this.nGridWMax;
				var ty = Math.floor(i / this.nGridWMax);

				this.aGrids[tx][ty] = type;
				this.oAstarGrid.setWalkableAt(tx, ty, (block == 1));
				
				this.aConnectType[tx][ty] = block;

				// 和编辑器对应吧 2位以上 (1,2位是游泳和浅水)
				// type = extraFlag;
				// if ( !(block & 0x1) )
				// {
				// var x : int = i % maxCols;
				// var y : int = i / maxCols;
				// m_Scene.aStarExGrids.setWalkable( x, y, false );
				// }
			}

			//地图区域类型填充
			var connectType = 1;
			for (var y = 0; y != this.nGridHMax; y++) {
				for (var x = 0; x != this.nGridWMax; x++) {
					if ( this.aConnectType[x][y] == 1 ) {
						this.aConnectType[x][y] = ++connectType;   // 1:连通区域
						this.TryConnectTypeFill({x:x, y:y}, connectType, this.DIROFFSET);
					}
				}
			}
			cc.log( "decode map data cost:" + (TimerUtil.GetTimer() - time));
			// initASGrids();
			// aStarPath = new CAStar();
			// pathFinder = new PathFinder(this);

			this.oPathFinderf = new AStarFinder();
			
			//Mgr.gameSoundMgr.SetSceneSound( soundData );// 设置场景音乐
			
			//oPathFinderf = new PathFinderFast(this);
			return true;
		},

		Clear:function(){
			//oPathFinderf.dispose();
			this.oPathFinderf = null;
			this.oDebugPath = null;
			this.nCurMapResId = 0;
			this.nWidth = 0;
			this.nHeight = 0;
			this.nGridHMax = 0;
			this.nGridWMax = 0;
		},
		
		FixBlockPoint2:function(end){
			if( this.GridIsWork(end.x, end.y, true) )
				return end;
			for (var i = 0; i < 100; ++i){
				var dir = i % 8 ;
				var step = parseInt( i / 8 ) + 1;
				var curx = this.DIROFFSET[dir][0] * step + end.x;
				var cury = this.DIROFFSET[dir][1] * step + end.y;

				if (curx >= 0 && cury >= 0 && curx < this.nGridWMax && cury < this.nGridHMax && this.GridIsWork(curx, cury, true))
					return cc.p(curx, cury);
			}
			return end;
		},

		// / 修正阻挡点寻找某个方向的非阻挡点
		FixBlockPoint:function(start, end){
			var dir = MDirection.GetDir(end, start);
			if (MDirection.IsValid(dir) == false)
				dir = MDirection.UP;

			//cc.log("this.aConnectType[start.x][start.y]:" + this.aConnectType[start.x][start.y] + "    this.aConnectType[end.x][end.y]:" + this.aConnectType[end.x][end.y])

			var startType = this.aConnectType[start.x][start.y];
			if (this.aConnectType[end.x][end.y] == startType)
				return end;

			var i = 0;
			var j = 0;
			var ret = new cc.p();
			var testDir = 0;
			// 单方向发散查找 10次还找不到的话= = 就不管了，让寻路处理 ty
			var ret2 = new cc.p();
			for (var i = 0; i < 10; i++){
				var obDir = MDirection.GetObDir(dir);
				var exDir1 = MDirection.GetOffDir(dir, 2);
				var exDir2 = MDirection.GetOffDir(dir, -2);
				// 前方 ty
				ret.x = end.x + this.DIROFFSET[dir][0] * (i + 1);
				ret.y = end.y + this.DIROFFSET[dir][1] * (i + 1);
				if (this.GridIsWork(ret.x, ret.y, true) && this.aConnectType[ret.x][ret.y] == startType)
					return ret;

				testDir = dir;
				for (j = 0; j < (i + 1); j++)
				{
					testDir = MDirection.GetOffDir(dir, -3);
					ret2.x = ret.x + this.DIROFFSET[testDir][0] * (j + 1);
					ret2.y = ret.y + this.DIROFFSET[testDir][1] * (j + 1);
					if (this.GridIsWork(ret2.x, ret2.y, true) && this.aConnectType[ret2.x][ret2.y] == startType)
						return ret2;

					testDir = MDirection.GetOffDir(dir, 3);
					ret2.x = ret.x + this.DIROFFSET[testDir][0] * (j + 1);
					ret2.y = ret.y + this.DIROFFSET[testDir][1] * (j + 1);
					if (this.GridIsWork(ret2.x, ret2.y, true) && this.aConnectType[ret2.x][ret2.y] == startType)
						return ret2;
				}

				// 旁边1 ty
				ret.x = end.x + this.DIROFFSET[exDir1][0] * (i + 1);
				ret.y = end.y + this.DIROFFSET[exDir1][1] * (i + 1);
				if (this.GridIsWork(ret.x, ret.y, true) && this.aConnectType[ret.x][ret.y] == startType)
					return ret;

				testDir = exDir1;
				for (j = 0; j < i; j++)
				{
					testDir = MDirection.GetOffDir(exDir1, -3);
					ret2.x = ret.x + this.DIROFFSET[testDir][0] * (j + 1);
					ret2.y = ret.y + this.DIROFFSET[testDir][1] * (j + 1);
					if (this.GridIsWork(ret2.x, ret2.y, true) && this.aConnectType[ret2.x][ret2.y] == startType)
						return ret2;

					testDir = MDirection.GetOffDir(exDir1, 3);
					ret2.x = ret.x + this.DIROFFSET[testDir][0] * (j + 1);
					ret2.y = ret.y + this.DIROFFSET[testDir][1] * (j + 1);
					if (this.GridIsWork(ret2.x, ret2.y, true) && this.aConnectType[ret2.x][ret2.y] == startType)
						return ret2;
				}

				// 旁边2 ty
				ret.x = end.x + this.DIROFFSET[exDir2][0] * (i + 1);
				ret.y = end.y + this.DIROFFSET[exDir2][1] * (i + 1);
				if (this.GridIsWork(ret.x, ret.y, true) && this.aConnectType[ret.x][ret.y] == startType)
					return ret;

				testDir = exDir2;
				for (j = 0; j < i; j++)
				{
					testDir = MDirection.GetOffDir(exDir2, -3);
					ret2.x = ret.x + this.DIROFFSET[testDir][0] * (j + 1);
					ret2.y = ret.y + this.DIROFFSET[testDir][1] * (j + 1);
					if (this.GridIsWork(ret2.x, ret2.y, true) && this.aConnectType[ret2.x][ret2.y] == startType)
						return ret2;

					testDir = MDirection.GetOffDir(exDir2, 3);
					ret2.x = ret.x + this.DIROFFSET[testDir][0] * (j + 1);
					ret2.y = ret.y + this.DIROFFSET[testDir][1] * (j + 1);
					if (this.GridIsWork(ret2.x, ret2.y, true) && this.aConnectType[ret2.x][ret2.y] == startType)
						return ret2;
				}

				// 后方 ty
				ret.x = end.x + this.DIROFFSET[obDir][0] * (i + 1);
				ret.y = end.y + this.DIROFFSET[obDir][1] * (i + 1);
				if (this.GridIsWork(ret.x, ret.y, true) && this.aConnectType[ret.x][ret.y] == startType)
					return ret;

				testDir = obDir;
				for (j = 0; j < (i + 1); j++)
				{
					testDir = MDirection.GetOffDir(obDir, -3);
					ret2.x = ret.x + this.DIROFFSET[testDir][0] * (j + 1);
					ret2.y = ret.y + this.DIROFFSET[testDir][1] * (j + 1);
					if (this.GridIsWork(ret2.x, ret2.y, true) && this.aConnectType[ret2.x][ret2.y] == startType)
						return ret2;

					testDir = MDirection.GetOffDir(obDir, 3);
					ret2.x = ret.x + this.DIROFFSET[testDir][0] * (j + 1);
					ret2.y = ret.y + this.DIROFFSET[testDir][1] * (j + 1);
					if (this.GridIsWork(ret2.x, ret2.y, true) && this.aConnectType[ret2.x][ret2.y] == startType)
						return ret2;
				}
			}

			return end;
		},

		IsSameConnectType:function(start, end){
			return (this.aConnectType[end.x][end.y] == this.aConnectType[start.x][start.y])
		},

		TryConnectTypeFill:function (spt, type,diroffy) {

			var list = {0:spt};
			var len = 1;
			var j = 0;
			var maxX = this.nGridWMax;
			var maxY = this.nGridHMax;
			while( j < len ) {
				var start = list[j];
				//向各个方向发散
				for (var i = 0; i < MDirection.NUM; ) {
//					var check = new cc.p();
					var checkX = start.x + diroffy[i][0];
					var checkY = start.y + diroffy[i][1];
					if( checkX > 0 && checkX < maxX && checkY > 0 && checkY < maxY ){
						if ( this.aConnectType[checkX][checkY] == 1 ) {
							this.aConnectType[checkX][checkY] = type;   // 1:连通区域
							list[len] = {x:checkX, y:checkY};
							len++;
						}
					}
					i = i + 2;
				}
				j++;
			}
			
		},


		// / 洪水蔓延尝试填充
		TryFloodFill:function(pt, expt, curfNum, maxfNum, diroffv){
			if (ToolBase.IsEqualPoint(expt, pt))
				return maxfNum;

			if (pt.x >= 0 && pt.y >= 0 && pt.x < this.nGridWMax && pt.y < this.nGridHMax && this.GridIsWork(pt.x, pt.y, true))
				curfNum++;

			if (curfNum >= maxfNum)
				return maxfNum;

			for (var i = 0; i < MDirection.NUM; i++){
				var checkx = pt.x + diroffv[i][0];
				var checky = pt.y + diroffv[i][1];
				if (checkx >= 0 && checky >= 0 && checkx < this.nGridWMax && checky < this.nGridHMax && this.GridIsWork(checkx, checky, true)){
					curfNum = this.TryFloodFill(cc.p(checkx, checky), expt, curfNum, maxfNum, diroffv);
					if (curfNum >= maxfNum){
						break;
					}
				}
			}

			return curfNum;
		},

		TestTargetBound:function(pt, expt){
			var tryfnum = this.TryFloodFill(pt, expt, 0, 8, this.DIROFFSET);
			if (tryfnum < 8)
				return true;

			return false;
		},
		
		// A*寻路，返回一组格子坐标
		AstarFindPath:function(sx, sy, ex, ey ){
//			var s = new Date().getTime();
			this.oAstarGrid.clear();
			var retPath = this.oPathFinderf.findPath(sx, sy, ex, ey, this.oAstarGrid);
//			var e = new Date().getTime() - s;
//			cc.log( "use time:" + e );
			return retPath;
// 			if (retPath == null || retPath.length == 0)
// 				return null;
// 			return Util.smoothenPath(this.oAstarGrid, retPath);		//平滑放到上层处理
		},

		// 返回路径，第一个点为起始点，注意返回的非Point而是数组01，路径为精简路径，非全点集
		FindPaths:function(start, end){
			end = this.FixBlockPoint(start, end);
			var retList = null;
			this.oEndpt.x = end.x;
			this.oEndpt.y = end.y;
			if (this.TestTargetBound(end, start) == false){
				var find = this.AstarFindPath(start.x, start.y, end.x, end.y);
				retList = find;
			}
			return retList;
		},

		/**
		 * 这里先用简化版 		暂时不用 Util.interpolate
		 * @param start
		 * @param end
		 * @param distance
         * @constructor
         */
		FindFlyPath:function(start, end, distance){
			distance = distance || 8;
			var x0 = start.x;
			var y0 = start.y;
			var x1 = end.x;
			var y1 = end.y;

			var abs = Math.abs,
				line = [],
				sx, sy, dx, dy, err, e2;

			dx = abs(x0 - x1);
			dy = abs(y0 - y1);

			sx = (x0 < x1) ? 1 : -1;
			sy = (y0 < y1) ? 1 : -1;

			err = dx - dy;

			while (true) {
				if(ToolMap.CalcDistance(start, cc.p(x0,y0)) > distance || !this.GridIsFly(x0,y0) )
					break;
				line.push([x0, y0]);

				if (x0 === x1 && y0 === y1) {
					break;
				}

				e2 = 2 * err;
				if (e2 > -dy) {
					err = err - dy;
					x0 = x0 + sx;
				}
				if (e2 < dx) {
					err = err + dx;
					y0 = y0 + sy;
				}
			}

			return line;
		},

		IsClear:function(){
			return (this.nCurMapResId == 0);
		},

		GetGridWidth:function(){
			return this.nGridWMax;
		},

		GetGridHeight:function(){
			return this.nGridHMax;
		},

		GetGridData:function(){
			return this.aGrids; 
		},

		// //////////////////////////////////////////////////////////////////////////////////
		// 格子类型
		// //////////////////////////////////////////////////////////////////////////////////
		Type:function(x, y){
			// if (this.aGrids[x] == undefined)
			// 	zz.Error("Type out 1!!" + x + "," + y);
			// else if (this.aGrids[x][y] == undefined)
			// 	zz.Error("Type out 2!!" + x + "," + y)
			if( x < 0 || y < 0 || x >= this.nGridWMax || y >= this.nGridHMax )
				return EnumSceneGridType.BARRIER;
			return parseInt(this.aGrids[x][y]);
		},
		
		TypeDy:function(x, y){
			if( x < 0 || y < 0 || x >= this.nGridWMax || y >= this.nGridHMax )
				return EnumSceneGridType.BARRIER;
			return parseInt(this.aGridsDynamic[x][y]);
		},
		
		TypeEx:function(x, y){
			if (this.aGridsExtra == null)
				return 0;
		
			return this.aGridsExtra[x][y];
		},
		
		GetExtraType:function(x, y){
			return this.TypeEx(x, y);
		},
		
		GridTypeIsWork:function(type){
			return ((type & EnumSceneGridType.WORK) != 0);
		},

		GridTypeIsFly:function(type){
			return ((type & EnumSceneGridType.FLY) != 0);
		},
		
		GridTypeIsWorkDy:function(type){
			return ((type & EnumSceneGridType.W_UF) == 0);
		},
		
		GridTypeIsCov:function(type){
			return ((type & EnumSceneGridType.COV) != 0);
		},
		
		GridTypeIsStop:function(type){
			return ((type & EnumSceneGridType.USTOP) == 0);
		},
		
		// //////////////////////////////////////////////////////////////////////////////////
		// 格子检测
		// //////////////////////////////////////////////////////////////////////////////////
		
		GridIsWork:function(x, y, isCheckObj){
			var nag = arguments.length;
			if (nag < 3)
				isCheckObj = true;
			
			var ret = this.GridTypeIsWork(this.Type(x, y));
			if (ret)
				ret = this.GridTypeIsWorkDy(this.TypeDy(x, y));
			
			if (!isCheckObj)
				return ret;
			
			return ret && this.CanMarkIsWalk(x, y);
		},

		GridIsFly:function(x, y, isCheckObj){
			return this.GridTypeIsFly(this.Type(x, y));
		},
		
		IsGridCanMove:function(x, y){
			return this.GridIsWork(x, y, true);
		},
		
		GetG:function(x, y){
			if (this.GridIsWork(x, y, true) == false)
				return 0;

			return 1;
		},
		
		GridIsWorkWithTrans:function(x, y, tx, ty, isCheckObj){
			var nag = arguments.length;
			if (nag < 5)
				isCheckObj = true;
			
			if (Math.abs(x - tx) > 4 || Math.abs(y - ty) > 4){
			 // 避开传送点
				if (this.GridIsTrans(x, y))
					return false;
			}
			
			return this.GridIsWork(x, y, isCheckObj);
		},
		
		GridIsTrans:function(x, y){
			var list = this.GetMark(x, y);
			if (list == null)
				return false;
			
			for (var i = list.length - 1; i >= 0; --i){
				if (list[i].IsTransGrid() && list[i].IsTransBlock())
				return true;
			}
			
			return false;
		},
		
		GridIsCov:function(x, y){
			var ret = this.GridTypeIsCov(this.Type(x, y));
			if (ret == false)
				ret = this.GridTypeIsCov(this.TypeDy(x, y));
			return ret;
		},
		
		GridIsWater:function(x, y){
			return ((this.Type(x, y) & EnumSceneGridType.WATER)) != 0;
		},
		
		GridIsWaterless:function(x, y){
			return ((this.Type(x, y) & EnumSceneGridType.WATERLESS)) != 0;
		},
		
		GridIsBAR:function(x, y, isCheckObj){
			var nag = arguments.length;
			if (nag < 3)
				isCheckObj = true;
		 
			var ret = this.GridTypeIsWork(this.Type(x, y));
			if (!isCheckObj)
				return !ret;
		
			return (!ret || !this.CanMarkIsWalk(x, y));
		},
		
		GridIsStop:function(x, y){
			return this.GridTypeIsStop(this.Type(x, y));
		},
		
		GridIsSafeable:function(x, y){
			return (this.Type(x, y) & EnumSceneGridType.SAFE) != 0;
		},

		 // //////////////////////////////////////////////////////////////////////////////////
		// 动态格子属性
		// //////////////////////////////////////////////////////////////////////////////////
		
		SetDyCoving:function(x, y, b){
			if (x < 0 || x >= this.nGridWMax || x < 0 || y >= this.nGridHMax)
				return;
		
			if (b)
				this.aGridsDynamic[x][y] = (this.aGridsDynamic[x][y] | EnumSceneGridType.COVERING) & 0x0ff;
			else
				this.aGridsDynamic[x][y] = (this.aGridsDynamic[x][y] & ~EnumSceneGridType.COVERING) & 0x0ff;
		},
		
		SetDyBlock:function(x, y, b){
			if (x < 0 || x >= this.nGridWMax || x < 0 || y >= this.nGridHMax)
				return;
		
			if (b)
				this.aGridsDynamic[x][y] = (this.aGridsDynamic[x][y] | EnumSceneGridType.W_UF) & 0x0ff;
			else
				this.aGridsDynamic[x][y] = (this.aGridsDynamic[x][y] & ~EnumSceneGridType.W_UF) & 0x0ff;
		},
		
		ClearDyAllMark:function(){
			for (var i = this.nGridHMax - 1; i >= 0; --i){
				for (var j = this.nGridWMax - 1; j >= 0; --j){
					this.aGridsDynamic[j][i] = 0;
				}
			}
		},
		
		
		// //////////////////////////////////////////////////////////////////////////////////
		// 格子对象列表
		// //////////////////////////////////////////////////////////////////////////////////
		
		SetMark:function(oldx, oldy, newx, newy, check){
			if (oldx >= 0 && oldy >= 0 && oldx < this.nGridWMax && oldy < this.nGridHMax){
				var list = this.aObjListGrids[oldx][oldy];
				if (list != null){
					for (var i = (list.length - 1); i >= 0; --i){
						if (list[i].gid == check.gid){
							list.splice(i, 1);
							break;
						}
					}
				}
			}
		
			if (newx >= 0 && newy >= 0 && newx < this.nGridWMax && newy < this.nGridHMax){
				var list2 = this.aObjListGrids[newx, newy];
				if (list2 == null){
					list2 = [];
					this.aObjListGrids[newx][newy] = list;
				}
		
				for (var i = (list2.length - 1); i >= 0; --i){
					if (list[i].gid == check.gid)
						return;
				}
		
				list.push(check);
			}
		},
		
		AddMark:function(x, y, check){
			var list = thisaObjListGrids[x][y];
			if (list == null){
				list = [];
				this.aObjListGrids[x][y] = list;
			}
		
			for (var i = list.length - 1; i >= 0; --i){
				if (list[i].gid == check.gid)
					return;
			}
		
			list.push(check);
		},
		
		RemoveMark:function(x, y, check){
			var list = this.aObjListGrids[x][y];
			if (list != null){
				for (var i = list.length - 1; i >= 0; --i){
					if (list[i].gid == check.gid){
						list.splice(i, 1);
						break;
					}
				}
			}
		},
		
		GetMark:function(x, y){
			return this.aObjListGrids[x][y];
		},
		
		CanMarkIsWalk:function(x, y, checkDead){
			var list = this.GetMark(x, y);
			if (list == null)
				return true;
		
			var checkd = false;
			var numargs = arguments.length;
			if (numargs >= 3)
				checkd = checkDead;
		
			for (var i = 0; i < list.length; ++i){
				var it = list[i];
				var isBolck = false;
				if (this.bMarkMonsterBolck && it.IsMonsterBlock())
					isBolck = true;
				else if (this.bMarkPetBolck && it.IsPetBlock())
					isBolck = true;
				else if (this.bMarkPlayerBolck && it.IsPlayerBlock())
					isBolck = true;
				else if (this.bMarkNpcBolck && it.IsNpcGrid())
					isBolck = true;
		
				if (this.bMarkCheckDead && isBolck && It.isDead())
					isBolck = false;
		
				if (isBolck)
					return false;
			}
			return true;
		},
		
		CheckMarkHaveObjcet:function(x, y, exceptDead, type){
			var numargs = arguments.length;
			var ecpd = true;
			var tp = 0;
		
			if (numargs < 4)
				type = 0;
			if (numargs < 3)
				exceptDead = true;
		
			var list = this.GetMark(x, y);
			if (list != null){
				for (var i = 0; i < list.length; ++i){
					var it = list[i];
					if ((type & 0x2) != 0 && it.IsMonsterGrid() && (!exceptDead || it.IsDead() == false))
						return true;
		
					if ((type & 0x1) != 0 && it.IsPlayerGrid() && (!exceptDead || it.IsDead() == false))
						return true;
		
					if ((type & 0x4) != 0 && it.IsNpcGrid() && (!exceptDead || it.IsDead() == false))
						return true;
		
					if ((type & 0x8) != 0 && it.IsPetGrid() && (!exceptDead || it.IsDead() == false))
						return true;
				}
			}
		
			return false;
		},
		
		// //////////////////////////////////////////////////////////////////////////////////
		// 格子调试
		// //////////////////////////////////////////////////////////////////////////////////
		
		/*
		 * private void initASGrids() { asGrids = new AStarGrid( gridRowMax,
		 * gridColMax );
		 * 
		 * for ( int x = 0; x < gridRowMax; x++ ) { for ( int y = 0; y <
		 * gridColMax; y++ ) { if ( aGrids[x, y] == 0 ) asGrids.setWalkable( x,
		 * y, false ); } }
		 * 
		 * asGrids.calculateLinks(0); }
		 * 
		 * public List<Vector2> FindPaths( Vector2 star, Vector2 end ) { List<Vector2>
		 * retList = new List<Vector2>();
		 * 
		 * aStarPath.grid = asGrids; asGrids.setStartNode((int)star.x,
		 * (int)star.y); asGrids.setEndNode((int)end.x, (int)end.y);
		 * 
		 * if ( !aStarPath.findPath(0) ) { return null; }
		 * 
		 * foreach ( AStarNode node in aStarPath.path ) { retList.Add( new
		 * Vector2( node.x, node.y ) ); }
		 * 
		 * return retList; }
		 */
		
		DrawDebugPath:function(){
		// float scale = transform.localScale.x;
		//
		// if (oEndpt.x != -1 && oEndpt.y != -1)
		// {
		// for (int i = 1; i < 15; ++i)
		// {
		// float sx = oEndpt.x * MGameDefine.GRID_WIDTH * scale;
		// float sy = oEndpt.y * MGameDefine.GRID_HEIGHT * scale + (i * 2 *
		// scale);
		// float ex = sx + MGameDefine.GRID_WIDTH * scale;
		// float ey = sy;
		// Debug.DrawLine(new Vector3(sx, -sy), new Vector3(ex, -ey),
		// Color.white);
		// }
		// }
		//
		//
		// if (oDebugPath != null && oDebugPath.Count > 1)
		// {
		//
		// for (int i = 1; i < oDebugPath.Count; ++i)
		// {
		// float sx = oDebugPath[i - 1].x * MGameDefine.GRID_WIDTH * scale +
		// ((MGameDefine.GRID_WIDTH / 2) * scale);
		// float sy = oDebugPath[i - 1].y * MGameDefine.GRID_HEIGHT * scale +
		// ((MGameDefine.GRID_HEIGHT / 2) * scale);
		// float ex = oDebugPath[i].x * MGameDefine.GRID_WIDTH * scale +
		// ((MGameDefine.GRID_WIDTH / 2) * scale);
		// float ey = oDebugPath[i].y * MGameDefine.GRID_HEIGHT * scale +
		// ((MGameDefine.GRID_HEIGHT / 2) * scale);
		// Debug.DrawLine(new Vector3(sx, -sy), new Vector3(ex, -ey),
		// Color.blue);
		// }
		// }
		},
		
		DrawGrid:function(){
		// if (!bDebugMode)
		// return;
		//
		// float scale = transform.localScale.x;
		//
		// for (int y = 0; y < nGridHMax; ++y)
		// {
		// int lienH = y * MGameDefine.GRID_HEIGHT;
		//
		// float tx = transform.position.x;
		// float ty = -transform.position.y;
		// Debug.DrawLine(new Vector3(tx, -(ty + lienH * scale)), new Vector3(tx
		// + nWidth * scale, -(ty + lienH * scale)), Color.grey);
		// }
		//
		// for (int x = 0; x < nGridWMax; ++x)
		// {
		// int lienW = x * MGameDefine.GRID_WIDTH;
		//
		// float tx = transform.position.x;
		// float ty = -transform.position.y;
		//
		// Debug.DrawLine(new Vector3(tx + lienW * scale, -ty, 0), new
		// Vector3(tx + lienW * scale, -(ty + nHeight * scale)), Color.grey);
		// }
		//
		// for (int x = 0; x < nGridWMax; ++x)
		// {
		// for (int y = 0; y < nGridHMax; ++y)
		// {
		// debugDridGrid(x, y, scale);
		// }
		// }
		
		 // (new Vector3(0, 0, 0), new Vector3(11, 11, 0), Color.red);
		},
		
		DebugDridGrid:function(xxx, yyy, scale){
		 /*
			 * for (int i = 0; i < 8; i++) { if ((type(xxx, yyy) & (i << i)) !=
			 * 0) { float sx = xxx * GameDefine.GRID_WIDTH * scale + (2 + i * 5) *
			 * scale; float sy = yyy * GameDefine.GRID_HEIGHT * scale + 15 *
			 * scale; float ex = sx + 3 * scale; float ey = sy;
			 * 
			 * Debug.DrawLine( new Vector3( sx, -sy, 0 ), new Vector3( ex, -ey,
			 * 0 ), new Color(25 + i * 25, 0, 0)); } }
			 */
		
		
// if ((type(xxx, yyy) & EnumSceneGridType.COV) != 0)
// {
// float sx = xxx * MGameDefine.GRID_WIDTH * scale;
// float sy = yyy * MGameDefine.GRID_HEIGHT * scale + (5 * scale);
// float ex = sx + MGameDefine.GRID_WIDTH * scale;
// float ey = sy + MGameDefine.GRID_HEIGHT * scale - (10 * scale);
//
// Debug.DrawLine(new Vector3(sx, -sy, 0), new Vector3(ex, -ey, 0),
// Color.yellow);
// }
//
// if ((type(xxx, yyy) & EnumSceneGridType.WORK) == 0)
// {
// float sx = xxx * MGameDefine.GRID_WIDTH * scale;
// float sy = yyy * MGameDefine.GRID_HEIGHT * scale;
// float ex = sx + MGameDefine.GRID_WIDTH * scale;
// float ey = sy + MGameDefine.GRID_HEIGHT * scale;
//
// Debug.DrawLine(new Vector3(sx, -sy, 0), new Vector3(ex, -ey, 0), Color.red);
// }
		}
};