﻿/**
 * 移动策略，用于执行在场景中移动控制
 * yq 2015年9月19日
 * ty 唯一可以修改场景对象xy的管理器
 */

var MMoveRule = cc.Class.extend( {

	// 控制的主对象
	_object:null,

	// 移动的路径列表(数组),改数据为简化后数组，只存储关键拐点坐标
	_pathList:null,

	// 上一次执行移动的时间
	_lastRunTime:0,

	// 移动速度
	_moveSpeed:210,

	_bCache:false,

	nCurMoveActionType:0,

	// 移动初始方向
	nStartDir:-1,
	// 跳跃距离
	nJumpDistance:0,
	// 跳跃起始高度
	nJumpStartHeight:0,
	// 跳跃起始位置
	oJumpStartPos:null,
	// 跳跃终点位置
	oJumpEndPos:null,
	// 当前跳跃高度
	nCurJumpHeight:0,
	jumpSpeed:0,
	_flyCount:0,

	ctor:function( obj ){
		this._object = obj;
	},

	/**
	 * 执行移动路径
	 * 参数paths:改数据为简化后数组，只存储关键拐点坐标,单个数据结构为{ x:0, y:0 }
	 * */
	Run:function( paths ){
		if (this._bCache)
			return;

		if ( paths == null || paths.length == 0 )
			return;

		this.nCurMoveActionType = EnumActionType.RUN;
		this._InitPath(paths);
		this._object.SetObjectHeightPos(0);
		this._lastRunTime = ToolBase.GetTimer();
		this._object.OnMoveBegin(this.nStartDir, this.nCurMoveActionType);
	},

	/**
	 * 跳跃
	 * @param paths
	 * @param isdouble
	 */
	Jump:function(paths, isFly){
		if (this._bCache)
			return;

		if (paths == null || paths.length != 2){
			zz.LogError("Moverule Jump path error !!!!!");
			return;
		}

		var isdouble = false;
		if (this.nCurMoveActionType == EnumActionType.JUMP_SKY2){
			// 临时底层限制最多二跳
			return;
		} else if (this.nCurMoveActionType == EnumActionType.JUMP){
			isdouble = true;
		}

		if( isFly ){
			this.nCurMoveActionType = EnumActionType.JUMP_FLY;
			this.nJumpStartHeight = 0;
			this._flyCount = 0;
		}
		else if (isdouble == true){
			this.nCurMoveActionType = EnumActionType.JUMP_SKY2;
			this.nJumpStartHeight = this.nCurJumpHeight;
		} else {
			this.nCurMoveActionType = EnumActionType.JUMP;
			this.nJumpStartHeight = 0;
		}

		this.nCurJumpHeight = 0;
		this._InitPath(paths);
		this.nJumpDistance = ToolMap.CalcDistance(this._pathList[0], this._pathList[1]);
		if( isFly ){
			var totalTime = this.nJumpDistance / MMoveRule.JUMP_MAX_SPEED + 0.1;	//服务器帧率问题可能导致前端到了服务器还差一帧 加0.1秒稳妥点
			MMoveRule.FLY_SPEED = parseInt( this.nJumpDistance / totalTime );
		}
		else {
			if( MMoveRule.JUMP_TIME > 0 ){
				this.jumpSpeed = this.nJumpDistance / ( MMoveRule.JUMP_TIME * 0.001 );
			}
		}
		this.oJumpStartPos = this._pathList[0];
		this.oJumpEndPos = this._pathList[1];
		this._pathList.shift();
		// 二跳不清高度
		if (isdouble != true) {
			this._object.SetObjectHeightPos(0);
		}
		this._lastRunTime = ToolBase.GetTimer();
		this._object.OnMoveBegin(this.nStartDir, this.nCurMoveActionType);
	},

	/**
	 * 坠崖
	 * @param paths
	 * @param isdouble
	 */
	DownHill:function(paths) {
		if (this._bCache)
			return;

		if (paths == null || paths.length != 2){
			zz.LogError("Moverule downhill path error !!!!!");
			return;
		}
		this._object.SetScale(1, 1);
		this.nCurMoveActionType = EnumActionType.DOWNHILL;
		this.nJumpStartHeight = 0;
		this.nCurJumpHeight = 0;
		this._InitPath(paths);
		this.oJumpStartPos = this._pathList[0];
		this.oJumpEndPos = this._pathList[1];
		this.nJumpDistance = ToolMap.CalcDistance(this._pathList[0], this._pathList[1]);

		this._object.SetObjectHeightPos(0);
		this._lastRunTime = ToolBase.GetTimer();
		this._object.OnMoveBegin(this.nStartDir, this.nCurMoveActionType);
		this._object.SetObjectHeightPos(0);
	},

	/**
	 * 瞬移
	 * 参数paths:改数据为简化后数组，只存储关键拐点坐标,单个数据结构为{ x:0, y:0 }
	 * */
	Fly:function( paths, mintime ){
		if (this._bCache)
			return;

		if ( paths == null || paths.length == 0 )
			return;

		this.nCurMoveActionType = EnumActionType.MOVEMOMENT;

		if( mintime ){
			this.nJumpDistance = ToolMap.CalcDistance(paths[0], this._object.GetPixPt());
			var totalTime = this.nJumpDistance / MMoveRule.JUMP_MAX_SPEED;
			MMoveRule.FLY_SPEED = parseInt( this.nJumpDistance / Math.max(totalTime, mintime) );
		}
		else{
			MMoveRule.FLY_SPEED = MMoveRule.JUMP_MAX_SPEED;
		}

		this._InitPath(paths);
		this._object.SetObjectHeightPos(0);
		this._lastRunTime = ToolBase.GetTimer();
		this._object.OnMoveBegin(this.nStartDir, this.nCurMoveActionType);
	},

	/**
	 * 击退
	 * 参数paths:改数据为简化后数组，只存储关键拐点坐标,单个数据结构为{ x:0, y:0 }
	 * */
	BackOut:function( paths ){
		if (this._bCache)
			return;

		if ( paths == null || paths.length == 0 )
			return;

		this.nCurMoveActionType = EnumActionType.BACKOUT;
		this._InitPath(paths);
		this._object.SetObjectHeightPos(0);
		this._lastRunTime = ToolBase.GetTimer();
		// 击退理论上不改变动作和方向
		// this._object.OnMoveBegin(this.nStartDir, this.nCurMoveActionType);
	},

	_InitPath:function(paths){
		this._pathList = paths;
		if (this.nCurMoveActionType == EnumActionType.DOWNHILL )
			this.nStartDir = ToolMap.CalcDirSlant(paths[1].x, paths[1].y, paths[0].x, paths[0].y);
		else if( this.nCurMoveActionType == EnumActionType.JUMP || this.nCurMoveActionType == EnumActionType.JUMP_FLY )
			this.nStartDir = ToolMap.CalcDirSlant(paths[0].x, paths[0].y, paths[1].x, paths[1].y);
		else
			this.nStartDir = ToolMap.CalcDirSlant(this._object.x, -this._object.y, paths[0].x, paths[0].y);

		for (var i = 0; i < this._pathList.length; ++i)
			this._pathList[i].y = -this._pathList[i].y;
	},

	/**
	 * 由上层调用每帧执行一次
	 * */
	OnFrame:function( curTime ){
		if ( this._pathList == null || this._pathList.length == 0 )
			return;

//		var pos = this._pathList.pop();
		var diffTime = curTime - this._lastRunTime;
		var oldPos = { x:this._object.x, y:this._object.y };

		var curSpeed = Math.min( this._moveSpeed, MMoveRule.MOVE_MAX_SPEED );
		if (this.nCurMoveActionType == EnumActionType.JUMP || this.nCurMoveActionType == EnumActionType.JUMP_SKY2)
		{
			if( this.jumpSpeed > 0 ){
				curSpeed = this.jumpSpeed;
			}
			else{
				var jumpRate = this.nCurMoveActionType == EnumActionType.JUMP ? MMoveRule.JUMP_SPEED_RATE[0] : MMoveRule.JUMP_SPEED_RATE[1];
				curSpeed = curSpeed * jumpRate;
			}
			curSpeed = Math.min( curSpeed, MMoveRule.JUMP_MAX_SPEED );
		}
		else if (this.nCurMoveActionType == EnumActionType.DOWNHILL)
			curSpeed = curSpeed * 1.5;
		else if (this.nCurMoveActionType == EnumActionType.MOVEMOMENT)
			curSpeed = MMoveRule.FLY_SPEED;
		else if(this.nCurMoveActionType == EnumActionType.JUMP_FLY)
			curSpeed = MMoveRule.FLY_SPEED;
		else if (this.nCurMoveActionType == EnumActionType.BACKOUT)
			curSpeed = MMoveRule.BACKOUT_SPEED;
		var dis = diffTime / 1000 * curSpeed;
		this._lastRunTime = curTime;

		var newPosition = null;
		var stop = false;
		var changeDir = false;
		while(this._pathList.length){
			var nextPos = this._pathList[0];
			// 计算两点距离
			var disnNext = Math.sqrt( Math.pow((oldPos.x - nextPos.x), 2) + Math.pow((oldPos.y - nextPos.y), 2) );
			if ( dis <= disnNext ) {
				var radius = Math.atan2( nextPos.y - oldPos.y, nextPos.x - oldPos.x );
				newPosition = { x:oldPos.x + dis * Math.cos(radius), y:oldPos.y + dis * Math.sin(radius) };
				if( changeDir ){
					var dir = ToolMap.CalcDirByAngle(oldPos.x, oldPos.y, nextPos.x, nextPos.y);
					this._object.SetDirection( dir );
				}
				break;		//end
			} else {
				oldPos = this._pathList.shift();
				if ( this._pathList.length == 0 ){ 	//end
					newPosition = oldPos;
					stop = true;
				}
				else{		//next path
					dis -= disnNext;
					changeDir = true;
				}
			}
		}

		var oldPos =  this._object.GetGridPt();
		// // 往服务器发送路径
		// if (this._object instanceof SceneMainPlayer) {
		// 	var paths = [];
		// 	paths[0] = cc.p(this._object.x, this._object.y);
		// 	paths[1] = cc.p(newPosition.x, newPosition.y);
		// 	cc.log("++++++++++++++++++++++++++++++++++++++++++++++++++++++" + this._object.x + " " + this._object.y + " " + newPosition.x + " " + newPosition.y);
		// 	Mgr.map.SendReqNewRunningMessage(this._object.x, this._object.y, paths);
		// }

		/// 判断意义何在？存在当x==0 y!=0 不会移动，予以修改
		//if ( newPosition.x && newPosition.y )
		if ( newPosition.x || newPosition.y )
		{
			this._object.x = newPosition.x;
			this._object.y = newPosition.y;
		}

		var gridPos = ToolMap.PixPt2GridPt(this._object.x, -this._object.y);

		if (this.nCurMoveActionType == EnumActionType.JUMP || this.nCurMoveActionType == EnumActionType.JUMP_SKY2 || this.nCurMoveActionType == EnumActionType.JUMP_FLY ){
			this._CalcJumpHeight();
			this._object.SetObjectHeightPos(this.nCurJumpHeight);
		} else if (this.nCurMoveActionType == EnumActionType.DOWNHILL) {
			this._CalcDownHillHeight();
			//zz.Log("curJumpHeight : " + this.nCurJumpHeight);
			this._object.SetObjectHeightPos(this.nCurJumpHeight);
		}

		if ( oldPos.x != gridPos.x || oldPos.y != gridPos.y ){
			this._object.OnMoveEnterGird(gridPos);
		}

		if ( stop )
			this.StopMove();
	},

	_CalcJumpHeight:function() {
		var maxDis = MMoveRule.JUMP_MAX_DISTANCE[0]
		var leaveDis = ToolMap.CalcDistance(cc.p(this._object.x, this._object.y), this.oJumpEndPos);
		var per = leaveDis / this.nJumpDistance;
		// if (per > 1)
		// 	per = 1;

		var partCount = Math.ceil(this.nJumpDistance / 1600);
		var partDis = Math.ceil( this.nJumpDistance / partCount );
		var partLeft = leaveDis % partDis;
		per = partLeft / partDis;

		var curCount = Math.ceil( leaveDis / partDis );
		if( curCount != 0 && curCount != this._flyCount )
		{
			if( this._flyCount != 0 )
				this._object.ChangeFlyAction();
			this._flyCount = curCount;
		}

		// 貌似对的，暂时先这样写吧
		var addv = Math.sin(Math.PI * per / 2);
		this.nCurJumpHeight = Math.sin(Math.PI * per) * Math.max(maxDis, partDis) * 0.2 + this.nJumpStartHeight * addv;

		//zz.Log("JUMPJUMP JUMPJUMP :" + this.nCurMoveActionType + " " + " curh:" + this.nCurJumpHeight + " per:" + per + " starth:" + this.nJumpStartHeight);
		//zz.Log("JUMPJUMP JUMPJUMP :" + addv + " " + this.nJumpStartHeight);
	},

	/**
	 * 坠崖，就跟跳跃一样吧，计算抛物线要变短
	 *
	 */
	_CalcDownHillHeight:function() {
		var leaveDis = ToolMap.CalcDistance(cc.p(this._object.x, this._object.y), this.oJumpEndPos);
		var passDis = (this.nJumpDistance - leaveDis);
		// 飞起高度
		var highDis = this.nJumpDistance * 0.25;
		if (passDis < highDis) {
			var per = (highDis - passDis) / highDis;
			if (per > 1)
				per = 1;
			this.nCurJumpHeight = Math.sin(Math.PI * per) * this.nJumpDistance * 0.2;
		} else {
			if (passDis < (highDis * 2)) {
				var scaledis = passDis - highDis;
				var perscale = (highDis * 2 - scaledis) / (highDis * 2);
				if (perscale > 1)
					perscale = 1;
				this._object.SetScale(perscale, perscale);
			}

			passDis -= highDis;
			var per = (((this.nJumpDistance - highDis) - passDis) / (this.nJumpDistance - highDis)) / 2 + 0.5;
			if (per > 1)
				per = 1;

			this.nCurJumpHeight = -Math.sin(Math.PI * per) * this.nJumpDistance * 2;
		}
		//zz.Log("JUMPJUMP JUMPJUMP :" + " curh:" + this.nCurJumpHeight + " per:" + per);
		//zz.Log("JUMPJUMP JUMPJUMP :" + addv + " " + this.nJumpStartHeight);
	},

	IsMoving:function(){
		return (this._pathList != null);
	},

	SetGridPt:function(pt){
		var ppt = ToolMap.GridPt2PixPt(pt.x, pt.y, true);
		this._object.x = ppt.x;
		this._object.y = -ppt.y;
	},

	SetPixPt:function(pt){
		this._object.x = pt.x;
		this._object.y = pt.y;
	},

	Clear:function(){
		this.nStartDir = 0;
		this.nJumpDistance = 0;
		this.nJumpStartHeight = 0;
		this.nCurJumpHeight = 0;

		this.nCurMoveActionType = EnumActionType.STAND;
		if (this._pathList != null)
			this._pathList = null;
		this._lastRunTime = 0;
		this.oJumpStartPos = null;
		this.oJumpEndPos = null;
	},

	OnRecycle:function(){
		this.Clear()
		this._bCache = true;
	},

	OnRevert:function(){
		this._bCache = false;
	},

	StopMove:function(notify){
		notify = ( notify != false );
		var beforeactiontype = this.nCurMoveActionType;
		this.Clear();
		if (this._object != null && notify) {
			this._object.OnMoveEnd(-1, beforeactiontype);
		}
		if (this._object != null) {
			if (beforeactiontype != EnumActionType.DOWNHILL)
				this._object.SetObjectHeightPos(0);
		}
	},

	SetMoveSpeed:function(s){
		this._moveSpeed = s;
	}

} );
MMoveRule.JUMP_SPEED_RATE = [2.0, 2.0];
MMoveRule.JUMP_MAX_DISTANCE = [800, 1200];
MMoveRule.JUMP_MAX_SPEED = 700;
MMoveRule.MOVE_MAX_SPEED = 700;
MMoveRule.FLY_SPEED = 700;
MMoveRule.BACKOUT_SPEED = 1000;
MMoveRule.JUMP_TIME = 0;
