﻿/**
 * 音频扩展，封装了DOM 和 webAudio
 * pbk 2015年11月4日
 */
var AudioEx = cc.Class.extend({
	/** 音频类型*/
	type_Audio: "AUDIO",
	/** 音频类型*/
	type_WebAudio: "WEBAUDIO",
    soundId:0,
	
	ctor: function( context, soundType, url ){
		var self = this;

		/** 音效ID*/
		self.id = 0;
		/** 音频使用类型*/
		self.soundType = soundType;
		/** 是否循环播放*/
		self.loop = false;
		/** 音频url地址*/
		self.url = url;
		self.src = url;

		self._playing = false;// 是否在播放状态中
		self._pause = false;// 是否在暂停状态中
		
		if ( context ){// Web Audio
			/** 音频类型*/
			self.type = self.type_WebAudio;
			
			self._channelSeparation = (soundType === SoundType_Trace);// 分离声道
			
			// window.AudioContext || window.webkitAudioContext || window.mozAudioContext
			self._context = context;
			
			// 音量控制 GainNode  context.createGain
			if ( self._channelSeparation ){// 范围音效分左右声道
				self._lGain = context.createGain();
				self._rGain = context.createGain();
				
				self._lVolume = 0.05;// 左声道音量大小
				self._rVolume = 0.05;// 右声道音量大小
			} else {
				self._vGain = context.createGain();
				
				self._volume = 1;// 音量大小
			}
			
			self._buffer = null;// 音频资源
			self._currentSource = null;// 当前播放音频源   AudioBufferSourceNode  context.createBufferSource
			self._startTime = null;// 开始播放时间
			self._currentTime = null;// 当前播放时间
			
			self._manualLoop = false;// 手动循环。谷歌浏览器版本42既为true
			self._ignoreEnded = false;// 是否忽视播放结束
			
			self._setBufferCallback = null;// 设置buffer完成回调
		} else {// DOM Audio
			/** 音频类型*/
			self.type = self.type_Audio;
			
			self._volume = 1;// 音量大小
			
			self._element = null;// document.createElement("audio")
		}
	},
	
	/**
	 * 播放音频
	 * @param audio 自定义音乐类
	 * @param loop 是否循环播放
	 * @param endCB 单次播放完毕回调（建议在非循环播放时使用）{cb:function, target:object, args:object}
	 */
	Play: function( audio, loop, endCB ){
        if ( !audio )
        {
            audio = this;
            loop = this.loop;

        }
		var volume = this.soundType == SoundType.BGM ? Mgr.gameSoundMgr.bgmVolume * Mgr.gameSoundMgr.bgmState : Mgr.gameSoundMgr.soundVolume;
		this.soundId = jsb.AudioEngine.play2d(audio.url, loop, volume);
		if( endCB )
			jsb.AudioEngine.setFinishCallback(this.soundId, function(){ endCB.cb.call( endCB.target, endCB.args ); });
	},
	/**182 0819 2204
	 * 停止播放音频
	 */
	Stop: function(){
        jsb.AudioEngine.stop(this.soundId);
    },
	/**
	 * 暂停播放
	 */
	Pause: function(){
		jsb.AudioEngine.pause(this.soundId);
	},
	/**
	 * 恢复播放
	 */
	Resume: function(){
		jsb.AudioEngine.resume(this.soundId);
	},
	/**
	 * 是否在播放状态中
	 * @returns {Boolean}
	 */
	GetPlaying: function(){
        return this.IsPlaying();
        /*
		if ( !this._playing) return false;
		var self = this;
		if ( self.type === self.type_Audio ){
			var audio = self._element;
			if ( !audio || self._pause || audio.ended ){// 没音效，暂停，播放完毕 都将返回false
				return self._playing = false;
			} else {
				return true;
			}
		} else {
			var sourceNode = self._currentSource;
			if ( !sourceNode || !sourceNode["playbackState"] ){
				return true;
			} else {
				return self._currentTime + self._context.currentTime - self._startTime < sourceNode.buffer.duration;
			}
		}
		*/
	},
	
	/**
	 * 设置音量
	 * 坑点：左右音道，设置音量为0时，可能会越界到负数，结果就两边都响了
	 * 坑点：音频资源需要注意，现在背景音乐可以正常调音
	 */
	SetVolume: function( volume ){
        jsb.AudioEngine.setVolume(this.soundId, volume);
		/*
		if ( self.type === self.type_Audio ){
			if ( volume > 1 ){ volume = 1;
			} else if ( volume < 0 ){ volume = 0; }
			
			self._volume = volume;
			
			if ( self._element ){
				self._element.volume = volume;
			}
		} else {
			if ( self._channelSeparation ){
				if ( volume > 1 ){ volume = 1;
				} else if ( volume < 0.05 ){ volume = 0.05; }
				
				self._lVolume = volume;
				
				if ( arguments.length >= 2 ){
					volume = arguments[1];
					
					if ( volume > 1 ){ volume = 1;
					} else if ( volume < 0.05 ){ volume = 0.05; }
				}
				
				self._rVolume = volume;
				
				if ( self._lGain ){
					self._lGain.gain.value = self._lVolume;
				}
				
				if ( self._rGain ){
					self._rGain.gain.value = self._rVolume;
				}
				
//				zz.Log( "左音量：" + self._lVolume + "  右音量：" + self._rVolume );
			} else {
				if ( volume > 1 ){ volume = 1;
				} else if ( volume < 0 ){ volume = 0; }
				
				self._volume = volume;
				
				if ( self._vGain ){
					self._vGain.gain.value = volume;
				}
			}
		}
		*/
	},
	
	/**
	 * 设置左声道音量
	 */
	SetLeftVolumef: function( volume ){
		var self = this;
        this.SetVolume(volume);

	},
	
	/**
	 * 设置右声道音量
	 */
	SetRightVolume: function( volume ){
		var self = this;
        this.SetVolume(volume);
	},

	/**
	 * 获取音量（不分声道才能使用）
	 * @returns {Number}
	 */
	GetVolume: function(){
        return jsb.AudioEngine.getVolume(this.soundId);
	},
	
	/**
	 * 获取左声道音量（分离双声道才能使用）
	 * @returns {Number}
	 */
	GetLeftVolume: function(){
        return this.GetVolume();
	},
	
	/**
	 * 获取右声道音量（分离双声道才能使用）
	 * @returns {Number}
	 */
	GetRightVolume: function(){
        return this.GetVolume();
	},
    IsPlaying:function () {
        return jsb.AudioEngine.getState(this.soundId);
    },
	/**
	 * 克隆音频
	 */
	CloneNode: function(){
		var self = this;
		var audio;
		if ( self.type === self.type_Audio){
			audio = new AudioEx();
           // audio._vGain=self._vGain;
			//var elem = document.createElement("audio");
            audio.src = self.src;
            audio.id = 0;
            /** 音频使用类型*/
            audio.soundType =  self.soundType;
            /** 是否循环播放*/
            audio.loop =  self.loop;
            /** 音频url地址*/
            audio.url =  self.url;
		} else {
			var volume = self._context["createGain"]();
			volume["gain"].value = 1;
			volume["connect"](self._context["destination"]);
			audio = new AudioEx( self._context, volume, self.src );
			
			if ( self._buffer ){
				audio.SetBuffer(self._buffer);
			} else {
				self._setBufferCallback = function(buffer){
					audio.SetBuffer( buffer );
					self._setBufferCallback = null;
				};
			}
			audio._manualLoop = self._manualLoop;
		}
		return audio;
	},

	// ==============================================================================================
	// ==================================================================================== webAudio
	// ==============================================================================================
	/**
	 * 设置webAudio数据
	 * @param buffer
	 */
	SetBuffer: function( buffer ){
		if ( !buffer ) return;
		
		var self = this;
		self.type = self.type_WebAudio;;
		
		var playing = self._playing;

		if ( self._buffer && self._buffer !== buffer && self.GetPlaying() ){
			self.Stop();// 当前有音频在播放中，且音频和传入数据不同，则暂停播放
		}
			
		self._buffer = buffer;
		
		if ( playing )// 若原在播放状态，则继续播放
			self.Play();
	},
	
	_PlayOfWebAudio: function( offset, endCB ){
//		zz.Log( "播放----------" );
		var self = this;
		
		var sourceNode = self._currentSource;
		if ( !self._buffer ){
			return;
		}
		
		if ( !self._pause && sourceNode ){// 未暂停且当前有播放资源
			if ( self._context.currentTime === 0 
				|| self._currentTime + self._context.currentTime - self._startTime > sourceNode.buffer.duration){
				self._StopOfWebAudio();
			} else {
				return;
			}
		}
		
		var audio = self._context.createBufferSource();// 创建一个音频源
		audio.buffer = self._buffer;// 填充音频数据
		
		if ( self._channelSeparation ){// 范围音效分左右声道
			var splitter = self._context.createChannelSplitter(2);
			audio.connect( splitter );
			var merger = self._context.createChannelMerger(2);

			splitter.connect( self._lGain, 0 );
			splitter.connect( self._rGain, 1 );

			self._lGain.connect( merger, 0, 0 );
			self._rGain.connect( merger, 0, 1 );

			merger.connect( self._context.destination );
			
			// 测试代码
			tAudio = self;
			tlG = self._lGain;
			trG = self._rGain;
			
			self.SetVolume( self._lVolume, self._rVolume );

		} else {
			audio.connect( self._vGain );
			
			self._vGain.connect( self._context.destination );
			
			self.SetVolume( self._volume );
		}
		
		audio.loop = (self._manualLoop ? false : self.loop);
		
		self._startTime = self._context.currentTime;
		self._currentTime = offset || 0;
		self._ignoreEnded = false;

		/*
		 * Safari on iOS 6 only supports noteOn(), noteGrainOn(), and noteOff() now.(iOS 6.1.3)
		 * The latest version of chrome has supported start() and stop()
		 * start() & stop() are specified in the latest specification (written on 04/26/2013)
		 *      Reference: https://dvcs.w3.org/hg/audio/raw-file/tip/webaudio/specification.html
		 * noteOn(), noteGrainOn(), and noteOff() are specified in Draft 13 version (03/13/2012)
		 *      Reference: http://www.w3.org/2011/audio/drafts/2WD/Overview.html
		 */
		if ( audio.start ){
			audio.start( 0, offset || 0 );
		} else if ( audio["noteGrainOn"] ){
			var duration = audio.buffer.duration;
			if ( self.loop ){
				/*
				 * On Safari on iOS 6, if loop == true, the passed in @param duration will be the duration from now on.
				 * In other words, the sound will keep playing the rest of the music all the time.
				 * On latest chrome desktop version, the passed in duration will only be the duration in this cycle.
				 * Now that latest chrome would have start() method, it is prepared for iOS here.
				 */
				audio["noteGrainOn"](0, offset, duration);
			} else {
				audio["noteGrainOn"](0, offset, duration - offset);
			}
		} else {
			// if only noteOn() is supported, resuming sound will NOT work
			audio["noteOn"](0);
		}
		
		self._currentSource = audio;
		
		audio["onended"] = function(){
			if ( self._manualLoop && self._playing && self.loop ){
				self.Stop();
				self.Play();
				return;
			}
			
			if ( self._ignoreEnded ){
				self._ignoreEnded = false;
			} else {
				if ( !self._pause )
					self.Stop();
				else
					self._playing = false;
			}
			
			if ( endCB ){
				endCB.cb.call( endCB.target, endCB.args );
			}
		};
	},
	
	_StopOfWebAudio: function(){
		var audio = this._currentSource;
		this._ignoreEnded = true;
		if ( audio ){
			audio.stop(0);
			this._currentSource = null;
		}
	},
	
	_PauseOfWebAudio: function(){
		this._currentTime += this._context.currentTime - this._startTime;
		var audio = this._currentSource;
		if ( audio ){
			audio.stop(0);
		}
	},
	
	_ResumeOfWebAudio: function(){
		var audio = this._currentSource;
		if ( audio ){
			this._startTime = this._context.currentTime;
			var offset = this._currentTime % audio.buffer.duration;
			this._PlayOfWebAudio(offset);
		}
	},


	// ==============================================================================================
	// ==================================================================================== Audio
	// ==============================================================================================
	/**
	 * 设置Audio数据
	 * @param element
	 */
	SetElement: function( element ){
		if ( !element ) return;
		
		var self = this;
		self.type = self.type_Audio;
		
		var playing = self._playing;

		if ( self._element && self._element !== element && self.GetPlaying() ){
			self.Stop();
		}

		self._element = element;
		
		if ( playing )
			self.Play();

		element.volume = self._volume;
		element.loop = self.loop;
	},

	_PlayOfAudio: function( offset, endCB ){
		var audio = this._element;
		if ( audio ){
			if ( endCB ){
				var onEnd = function(){
					audio.removeEventListener( "pause", onEnd, false );

					endCB.cb.call( endCB.target, endCB.args );
				};

				cc._addEventListener( audio, "pause", onEnd, false );
			}
			
			audio.loop = this.loop;
			audio.play();
		}
	},

	_StopOfAudio: function(){
		var audio = this._element;
		if ( audio ){
			audio.pause();
			if ( audio.duration && audio.duration !== Infinity )
				audio.currentTime = 0;
		}
	},

	_PauseOfAudio: function(){
		var audio = this._element;
		if ( audio ){
			audio.pause();
		}
	},
	
	_ResumeOfAudio: function(){
		var audio = this._element;
		if ( audio ){
			audio.play();
		}
	},
});