/**
 * UILabel控件，按钮控件
 */

var MUILabel = ccui.Widget.extend({
    ptype: "",
    _isHit: true,
    _textAlign: 0,
    uiHeight: 0,
    uiWidth: 0,
    _textArr: null,
    _htmlText: "",
    _htmlData: null,
    _color: null,
    _fontSize: 22,
    _fontName: null,
    _multiline: false,
    _textWidth: 0,
    _textHeight: 20,
    _rowCount: 1,
    _outlineColor: null,
    _outlineSize: 1,
    _isEnableOutline: false,		//默认不描边
    _leading: 3,

    ctor: function () {
        this.ptype = EnumPanelType.LABEL;
        this._super();
        this._textArr = [];
        this._htmlData = [];
        this._fontName = GameFont.DEFAULT;
        this._color = new cc.Color(255, 255, 255);
        this._outlineColor = new cc.Color();
        this.setAnchorPoint(0, 0);
    },

    clone:function(){
        var o = new MUILabel();
        o.topPanel=o;
        o.ParseJson( this.jsonobj );
        o.Init();
        return o;
    },

    Init: function () {
        var json = this.jsonobj;
        //解析LABEL数据		这里要移到UIParseUtil.ParseLabel里去
        UIParseUtil.ParseLabel(this, json);
        this.AddChildren()
    },
    /**
     * 设置描边颜色 by zfs
     * @param color:0xff00ff
     * @param lineWidth:1
     * @constructor
     */
    EnableOutline: function (color, size, alpha) {
        color = color || 0x0;
        size = size || 1;
        alpha = alpha || (size >= 2 ? 120 : 255);
        var newColor = UIUtil.GetCCColor(color, alpha);
        if (!this._isEnableOutline || !cc.colorEqual(newColor, this._outlineColor) || this._outlineSize != size) {
            this._outlineColor = newColor;
            this._outlineSize = size;
            for (var i = 0; i < this._textArr.length; i++) {
                this._textArr[i].enableOutline(this._outlineColor, size);
            }
        }
        this._isEnableOutline = true;
    },

    DisableEffect: function () {
        for (var i = 0; i < this._textArr.length; i++) {
            this._textArr[i].disableEffect();
        }
    },

    SetString: function (text) {
        text = String(text);	//这里转一下 免得比较的时候_htmlText会自动转换为int   	--merlin
        text = text.replace(/\r/g, "<br>");
        text = text.replace(/\n/g, "<br>");     //label是多个render组成  不能直接\n换行  必须从最左开始排列
        if (this._htmlText != text) {
            this._htmlText = text;
            this._ParseText();
        }
    },

    SetText: function (text) {
        this.SetString(text);
    },
    SetHtmlText: function (htmlText) {
        this.SetString(htmlText);
    },
    GetText: function () {
        return this.jsonobj.text;
    },
    GetHtmlText: function () {
        return this.jsonobj.htmlText;
    },
    GetString: function () {
        return this._htmlText;
    },
    FormatHtmlText:function(){
        this.SetHtmlText(TextUtil.FormatStr( this.GetHtmlText(), arguments ));
    },
    /**
     * 设置文本颜色
     * @param color:0xff00ff
     * @param alpha:255
     * @constructor
     */
    SetColor: function (color, alpha) {
        var newColor = UIUtil.GetCCColor(color, alpha);
        if (!cc.colorEqual(newColor, this._color)) {
            this._color = newColor;
            for (var i = 0; i < this._textArr.length; i++) {
                this._textArr[i].SetDefaultColor(this._color);
            }
        }
    },
    SetFontSize: function (size) {
        if (this._fontSize != size) {
            this._fontSize = size;
            for (var i = 0; i < this._textArr.length; i++) {
                this._textArr[i].SetFontSize(this._fontSize);
            }
            this._SortText();
        }
    },
    SetFontName: function (name) {
        if (this._fontName != name) {
            this._fontName = name;
            for (var i = 0; i < this._textArr.length; i++) {
                this._textArr[i].SetFontName(this._fontName);
            }
        }
    },

    SetIsHit: function (hit) {
        this._isHit = hit;
    },

    _ParseText: function () {
        var self = this;
        this._htmlData.length = 0;
        var data = this._htmlData;
        var colorArr = [];
        var color;
        var isUnderline = false;
        var isNewLine = false;
        var isLink = null;
        var strArr;
        while (strArr = TextUtil.HTML_SPILT_PATTERN.exec(this._htmlText)) {
            strArr.forEach(function (str, i) {
                if (i > 0 && str) {		//html做好标记 文本则创建text
                    var first = str.charAt(0);	//这里标签少 随便写了 不用正则也可以
                    if (first == '<') {
                        var second = str.charAt(1);
                        switch (second) {
                            case 'u':
                            case 'U':
                                isUnderline = true;
                                break;
                            case 'f':
                            case 'F':
                                var hColor = str.match(/["']#(.+?)["']/);
                                if (hColor)
                                    color = UIUtil.GetCCColor(parseInt(hColor[1], 16));
                                else
                                    color = self._color;
                                colorArr.push(color);
                                break;
                            case 'a':
                            case 'A':
                                var href = str.match(/href[ ="']+(.*)[ '"]+/);
                                isLink = href && href[1];
                                break;
                            case 'b':
                            case 'B':
                                if (str.charAt(2) == 'r' || str.charAt(2) == 'R')
                                    isNewLine = true;
                                data.push({newline:true});
                                break;
                            case '/':
                                var third = str.charAt(2);
                                switch (third) {
                                    case 'a':
                                    case 'A':
                                        isLink = null;
                                        break;
                                    case 'u':
                                    case 'U':
                                        isUnderline = false;
                                        break;
                                    case 'f':
                                    case 'F':
                                        colorArr.pop();
                                        color = colorArr.length > 0 ? colorArr[colorArr.length - 1] : null;
                                        break;
                                }
                                break;
                        }
                        return;	//forEach里return只是continue的作用而已
                    }
                    data.push({str: str, color: color, underline: isUnderline,isLink:isLink});
                    isNewLine = false;
                }
            });
        }
        if (this._htmlData.length == 0) {
            this._htmlData.push({str: this._htmlText, underline: isUnderline,isLink:isLink});
        }
        this._RenderText();
    },

    SetOpenUrlCallback:function(callback,target){
        this._callback = callback;
        this.__target = target;

    },

    _OnClicked:function(href,sender){
        if(this._callback){
            this._callback.call(this.__target,href);
        }
    },
    _RenderText: function () {	//等于是自己实现了个richtext -- merlin
        this.ClearText();
        var size = this._fontSize;
        var font = this._fontName;
        var leftW = this.uiWidth;
        var self = this;
        var data = this._htmlData;
        var costWidth = 0;
        var _textWidth = 0,_maxWidth = 0;
        for (var i = 0; i < data.length; i++) {
            var info = data[i];
            if( info.newline ){
                costWidth = 0;
                this._rowCount += 1;
                // 换行重置
                _maxWidth = Math.max(_maxWidth,_textWidth);
                _textWidth = 0;
                continue;
            }
            var str = info.str;
            var color = info.color;
            var isUnderline = info.underline;
            var render = LabelRender.Create();
            render.SetColor(color);
            render.SetDefaultColor(this._color);
            render.SetFontName(font);
            render.SetFontSize(size);
            render.SetText(str);
            render.SetUnderLine(isUnderline);

            // if(info.isLink && info.isLink != ""){
            //     render.setTouchEnabled(true);
            //     render.addClickEventListener(this._OnClicked.bind(this,info.isLink));
            // }else{
            //     render.setTouchEnabled(false);
            // }

            if (this._isEnableOutline)
                render.enableOutline(this._outlineColor, this._outlineSize);
            render.lineNum = this._rowCount;
            _textWidth += render.GetWidth();
            if (i == 0) {
                this._textHeight = render.GetHeight();
            }
            self._textArr.push(render);
            self.addChild(render);
            if (this._multiline) {
                while (render.GetWidth() + costWidth > leftW) {
                    // 减去本行已经加过的 --jaye
                    _textWidth -= render.GetWidth() ;
                    //ccui.Helper.getSubStringOfUTF8String();       TextUtil.GetCharactersByString
                    var realLen = Math.max(parseInt(leftW / render.GetWidth() * str.length), 1);	//最小也要保证为1 避免死循环
                    //数字和文字的大小不一样这里检查下吧
                    render.SetText(str.substr(0, realLen));
                    if(realLen>1) {
                        if( render.GetWidth() + costWidth > leftW ){
                            while (true) {
                                realLen--;
                                render.SetText(str.substr(0, realLen));
                                if (realLen == 1 || render.GetWidth() + costWidth <= leftW)
                                    break;
                            }
                        }
                        else{
                            while (render.GetWidth() + costWidth < leftW) {
                                realLen++;
                                render.SetText(str.substr(0, realLen));
                                if (render.GetWidth() + costWidth > leftW) {
                                    realLen--;
                                    render.SetText(str.substr(0, realLen));
                                    break;
                                }
                            }
                        }
                    }
                    else{
                        render.SetText(str.substr(0, realLen));
                    }
                    costWidth = 0;
                    // 重新加上本行没加的 --jaye
                    _textWidth += render.GetWidth();
                    //render.SetText(str.substr(0, realLen));
                    str = str.substr(realLen);
                    leftW = self.uiWidth;
                    this._rowCount += 1;
                    // 换行重置
                    _maxWidth = Math.max(_maxWidth,_textWidth);
                    _textWidth = 0;

                    render = LabelRender.Create();
                    render.SetColor(color);
                    render.SetDefaultColor(this._color);
                    render.SetFontName(font);
                    render.SetFontSize(size);
                    render.SetText(str);
                    render.SetUnderLine(isUnderline);
                    if (this._isEnableOutline)
                        render.enableOutline(this._outlineColor, this._outlineSize);
                    render.lineNum = this._rowCount;
                    self._textArr.push(render);
                    self.addChild(render);
                }
                costWidth += render.GetWidth();
                if (str[str.length - 1] == "\n") {
                    costWidth = 0;
                    this._rowCount++;
                    // 换行重置
                    _maxWidth = Math.max(_maxWidth,_textWidth);
                    _textWidth = 0;
                }
            }
        }

        // 去单行最大的作为文本宽度
        _maxWidth = Math.max(_maxWidth,_textWidth);
        this._textWidth = _maxWidth
        this._SortText();
    },

    /**
     * 每种情况都考虑有点复杂 先支持常用的 多行全按左对齐来处理
     * @private
     */
    _SortText: function () {
        // fix forget add this._leading,so the content will be lower than expected.
        //var startY = this.uiHeight ? this.uiHeight : this._rowCount * this._textHeight;
        var startY = this.GetHeight() + this._textHeight + this._leading;
        var startX = 0;

        var lineWidth = {};
        var row = -1;
        var render;
        for (var i = 0; i < this._textArr.length; i++) {
            render = this._textArr[i];
            if (row != render.lineNum) {
                row = render.lineNum;
                lineWidth[row] = 0;
            }
            lineWidth[row] += render.GetWidth();
        }

        var curX = startX;
        var row = -1;
        var w = this.uiWidth ? this.uiWidth : 0;
        for (var i = 0; i < this._textArr.length; i++) {
            render = this._textArr[i];
            if (row != render.lineNum) {
                curX = startX;
                row = render.lineNum;

                if(this._textAlign == 1){
                    curX = (w - lineWidth[row]) * 0.5;
                }else if (this._textAlign == 2){
                    curX = w - lineWidth[row];
                }
                startY = this.GetHeight() - ( render.lineNum - 1 ) * ( this._textHeight + this._leading );
            }
            render.x = curX;
            curX += render.GetWidth();
            render.y = startY;
        }
        delete lineWidth;
        this.width = this.GetWidth();
        this.height = this.GetHeight();
    },

    ClearText: function () {
        this._textArr.forEach(function (render) {
            LabelRender.Delete(render);
            if (render.parent)
                render.parent.removeChild(render);
        });
        this._textArr.length = 0;
        this._textWidth = 0;
        this._textHeight = 20;
        this._rowCount = 1;
    },

    SetTextAlign: function (align) {
        if (this._textAlign != align) {
            this._textAlign = align;
            this._SortText();
        }
    },

    SetMultiline: function (value) {
        if (this._multiline != value) {
            this._multiline = value;
            this._RenderText();
        }
    },

    GetWidth: function () {
        if (this.uiWidth)
            return this.uiWidth;
        return this._textWidth;
    },

    GetHeight: function () {
        if (this.uiHeight)
            return this.uiHeight;
        return this.GetTextHeight();
    },

    GetTextHeight:function(){
        return ( this._rowCount - 1 ) * ( this._textHeight + this._leading ) + this._textHeight;
    },

    SetUISize: function (w, h) {
        if (this.uiWidth != w || ( h && this.uiHeight != h )) {
            var shouldRender = false;
            if(this.uiWidth != w) shouldRender = true;
            this.uiWidth = w;
            if (h)
                this.uiHeight = h;
            if(shouldRender){
                this._RenderText();
            }else{
                this._SortText();
            }
        }
    },

    SetLeading: function (value) {
        if (this._leading != value) {
            this._leading = value;
            this._RenderText();
        }
    },
    GetRows: function () {
        return this._rowCount;
    },
    GetFontSize: function () {
        return this._fontSize;
    }
});