﻿/**
 * 二进制流装载器
 * 传入一个二进制数据块，可以方便的读取里边的二进制数据，如int,bool,string等
 * yq 2015年9月7日 
 * pbk 2015年11月2日 自动适应大小端读法
 *
 */
var BinStream = cc.Class.extend( {
	
	ctor:function( datas ){
		var self = this;
		
		self.SetData( datas );
		//这个应该是看是大端存入还是小端存入的
		self.endian = self.BIG_ENDIAN;
	},
	
	SetData:function( datas ){
		if ( datas == null )
			datas = [];

		this._baseData = datas;
		// 当前读取位置标记位
		this._pos = 0;
		// 剩余有效字节数
		this._bytesAvailable = datas.length;
	},
	
	length:function(){
		return this._baseData.length;
	},
	
	bytesAvailable:function(){
		return this._bytesAvailable;
	},
	
	WriteShort:function( val ){
		var tmpPos = this._pos;
		var perLen = this._baseData.length;

		if ( this.endian == this.BIG_ENDIAN ){
			this._baseData[tmpPos++] = 0xff & ( val >> 8 );
			this._baseData[tmpPos++] = 0xff & ( val );
		} else {
			this._baseData[tmpPos++] = 0xff & ( val );
			this._baseData[tmpPos++] = 0xff & ( val >> 8 );
		}
		
		this._pos += 2;
		this._bytesAvailable += ( this._baseData.length - perLen );
	},

	WriteUShort:function( val ){
		this.WriteShort(val);
	},
	
	WriteInt:function( val ){
		var tmpPos = this._pos;
		var perLen = this._baseData.length;
		
		if ( this.endian == this.BIG_ENDIAN ){
			this._baseData[tmpPos++] = 0xff & ( val >> 24 );
			this._baseData[tmpPos++] = 0xff & ( val >> 16 );
			this._baseData[tmpPos++] = 0xff & ( val >> 8 );
			this._baseData[tmpPos++] = 0xff & ( val );
		} else {
			this._baseData[tmpPos++] = 0xff & ( val );
			this._baseData[tmpPos++] = 0xff & ( val >> 8 );
			this._baseData[tmpPos++] = 0xff & ( val >> 16 );
			this._baseData[tmpPos++] = 0xff & ( val >> 24 );
		}
		
		this._pos += 4;
		this._bytesAvailable += ( this._baseData.length - perLen );
	},

	WriteVarInt:function( value ){
		var tmpPos = this._pos;
		var perLen = this._baseData.length;
		while (true) {
			// if ( this.endian == this.BIG_ENDIAN ) {	本身是自己一个个写的byte 所以不存在大小端的问题
				if ((value & ~0x7F) == 0) {
					this._baseData[tmpPos++] = value;
					this._pos++;
					break;
				} else {
					this._baseData[tmpPos++] = ((value & 0x7F) | 0x80);
					this._pos++;
					value >>>= 7;
				}
			// }
		}
		this._bytesAvailable += ( this._baseData.length - perLen );
	},

	WriteVarLong:function( value ){	//这个应该不会用到 前端不会向后端发这么大的数 还是先实现出来	--merlin
		var tmpPos = this._pos;
		var perLen = this._baseData.length;
		var bitStr;
		var hValue = Math.floor( value/Math.pow(2,32) );
		var lValue = value - hValue * Math.pow(2,32);
		var bitStr = ( hValue >>> 0 ).toString(2).concat( BinStream.ZERO_BIT.concat(lValue.toString(2)).substr(-32) );
		var len = 0;
		while(true){
			if( bitStr.length <= 7 ){
				this._baseData[tmpPos++] = parseInt(bitStr.substr(len-7, 7));
				break;
			}
			else{
				this._baseData[tmpPos++] = parseInt(bitStr.substr(len-7, 7))|0x80;
				len += 7;
			}
		}
		this._bytesAvailable += ( this._baseData.length - perLen );
	},
	
	WriteLong:function( val ){
		
		this.WriteBean(val);
//		var tmpPos = this._pos;
//		var perLen = this._baseData.length;
//		
//		if ( this.endian == this.BIG_ENDIAN ){
//			this._baseData[tmpPos++] = 0xff & ( val >> 56 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 48 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 40 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 32 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 24 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 16 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 8 );
//			this._baseData[tmpPos++] = 0xff & ( val );
//		} else {
//			this._baseData[tmpPos++] = 0xff & ( val );
//			this._baseData[tmpPos++] = 0xff & ( val >> 8 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 16 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 24 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 32 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 40 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 48 );
//			this._baseData[tmpPos++] = 0xff & ( val >> 56 );
//		}
		
//		this._pos += 8;
//		this._bytesAvailable += ( this._baseData.length - perLen );
	},
	
	WriteByte:function( val ){
		var perLen = this._baseData.length;
		this._baseData[this._pos++] = 0xff & ( val );
		this._bytesAvailable += ( this._baseData.length - perLen );
	},
	
	WriteBytes:function( bytes ){

		var perLen = this._baseData.length;
		this._baseData = this._baseData.concat( bytes );
		this._pos += bytes.length;
		this._bytesAvailable += ( this._baseData.length - perLen );
	},
	
	WriteString:function( str ){
		if ( str == null )
			this.WriteInt(0);
		else{
			var utf8Bytes = this.toUTF8Array(str);
			this.WriteInt(utf8Bytes.length);
			this.WriteBytes( utf8Bytes );
		}
	},
	
	WriteBean:function( object ){
		object.writing( this );
	},
	
	ReadShort:function(){
		var tmpPos = this._pos;
		this._pos += 2;
		this._bytesAvailable -= 2;

		var result;
		if ( this.endian == this.BIG_ENDIAN ){
			result = ( this._baseData[tmpPos++] << 8 ) | ( this._baseData[tmpPos++] );
		} else {
			result = ( this._baseData[tmpPos++] ) | ( this._baseData[tmpPos++] << 8 );
		}
		if( result > 32767)
			result -= 65536;
		return result;
	},

	ReadUShort:function(){
		var tmpPos = this._pos;
		this._pos += 2;
		this._bytesAvailable -= 2;

		if ( this.endian == this.BIG_ENDIAN ){
			return ( this._baseData[tmpPos++] << 8 ) | ( this._baseData[tmpPos++] );
		} else {
			return ( this._baseData[tmpPos++] ) | ( this._baseData[tmpPos++] << 8 );
		}
	},
	
	ReadInt:function(){
		var tmpPos = this._pos;
		this._pos += 4;
		this._bytesAvailable -= 4;
		
		if ( this.endian == this.BIG_ENDIAN ){
			return ( this._baseData[tmpPos++] << 24 ) |  
			( this._baseData[tmpPos++] << 16 ) |
			( this._baseData[tmpPos++] << 8 ) |
			( this._baseData[tmpPos++] );
		} else {
			return ( this._baseData[tmpPos++] ) |  
			( this._baseData[tmpPos++] << 8 ) |
			( this._baseData[tmpPos++] << 16 ) |
			( this._baseData[tmpPos++] << 24);
		}
		
	},

	ReadVarInt:function(){
		var tmpPos = this._pos;
		var count = 0;

		var tmp = this._baseData[tmpPos++];
		if (tmp < 128){
			count = tmpPos - this._pos;
			this._pos += count;
			this._bytesAvailable -= count;
			return tmp;
		}
		var result = tmp & 0x7f;
		if ((tmp = this._baseData[tmpPos++]) < 128)
			result |= tmp << 7;
		else
		{
			result |= (tmp & 0x7f) << 7;
			if ((tmp = this._baseData[tmpPos++]) < 128)
				result |= tmp << 14;
			else
			{
				result |= (tmp & 0x7f) << 14;
				if ((tmp = this._baseData[tmpPos++]) < 128)
					result |= tmp << 21;
				else
				{
					result |= (tmp & 0x7f) << 21;
					result |= (tmp = this._baseData[tmpPos++]) << 28;
					if (tmp < 0)	//这里不该小于0才对	--merlin
					{
						for (var i = 0; i < 5; i++)
						{
							if (this._baseData[tmpPos++] >= 0)
								return result;
						}
						throw new Error("Bean:readInt()");
					}
				}
			}
		}
		count = tmpPos - this._pos;
		this._pos += count;
		this._bytesAvailable -= count;
		return result;
	},

	ReadVarLong:function(){
		var shift = 0;
		var value = 0;
		var tmpPos = this._pos;
		var count = 0;

		while ( shift < 64 )
		{
			var b = this._baseData[tmpPos++];
			value += ( b & 0x7f ) * Math.pow( 2, shift );
			if ( (b&0x80) == 0 )
				break;

			shift +=  7;
		}

		count = tmpPos - this._pos;
		this._pos += count;
		this._bytesAvailable -= count;
		return value;
	},
	
		ReadUint:function(){
			var tmpPos = this._pos;
			this._pos += 4;
			this._bytesAvailable -= 4;

			if ( this.endian == this.BIG_ENDIAN ){
				return parseInt( this._baseData[tmpPos++].toString(16) + "000000", 16 ) +
					(( this._baseData[tmpPos++] << 16 ) |
					( this._baseData[tmpPos++] << 8 ) |
					( this._baseData[tmpPos++] ));
			} else {
				return (( this._baseData[tmpPos++] ) |
					( this._baseData[tmpPos++] << 8 ) |
					( this._baseData[tmpPos++] << 16 )) +
					parseInt( this._baseData[tmpPos++].toString(16) + "000000", 16 );
			}

	},
	
	ReadLong:function(){
		return this.ReadBean(Long);
//		var tmpPos = this._pos;
//		this._pos += 8;
//		this._bytesAvailable -= 8;
//		
//		if ( this.endian == this.BIG_ENDIAN ){
//			return ( this._baseData[tmpPos++] << 56 ) |  
//			( this._baseData[tmpPos++] << 48 ) |
//			( this._baseData[tmpPos++] << 40 ) |
//			( this._baseData[tmpPos++] << 32 ) |
//			( this._baseData[tmpPos++] << 24 ) |
//			( this._baseData[tmpPos++] << 16 ) |
//			( this._baseData[tmpPos++] << 8 ) |
//			( this._baseData[tmpPos++] );
//		} else {
//			return ( this._baseData[tmpPos++] ) |  
//			( this._baseData[tmpPos++] << 8 ) |
//			( this._baseData[tmpPos++] << 16 ) |
//			( this._baseData[tmpPos++] << 24 ) |
//			( this._baseData[tmpPos++] << 32 ) |
//			( this._baseData[tmpPos++] << 40 ) |
//			( this._baseData[tmpPos++] << 48 ) |
//			( this._baseData[tmpPos++] << 56 );
//		}
	},
	
	ReadByte:function(){		
		this._bytesAvailable -= 1;
		return ( this._baseData[this._pos++] );
	},
	
	/**
	 * 读取数据块
	 * */
	ReadBytes:function( startPos, len ){
		if ( len == null )
			len = startPos;
		
		var endPos = this._pos + len;		
		
		this._bytesAvailable -= len;
		var ret = this._getSubArray(this._baseData, this._pos, endPos);
		this._pos = endPos;
		return ret;
	},
	
	ReadString:function(){
		var len = this.ReadInt();
		if ( len == 0 )
			return null;
		
		var utf8Bytes = this.ReadBytes( len );
		return this.Utf8ArrayToStr( utf8Bytes );		
	},
	
	ReadBean:function( _class ){
		var bean = new _class();
		bean.reading( this );
		return bean; 
	},
	
	toUTF8Array:function(str) {
		str += "";
		var utf8 = [];
		for (var i=0; i < str.length; i++) {
			var charcode = str.charCodeAt(i);
			if (charcode < 0x80) utf8.push(charcode);
			else if (charcode < 0x800) {
				utf8.push(0xc0 | (charcode >> 6), 
						0x80 | (charcode & 0x3f));
			}
			else if (charcode < 0xd800 || charcode >= 0xe000) {
				utf8.push(0xe0 | (charcode >> 12), 
						0x80 | ((charcode>>6) & 0x3f), 
						0x80 | (charcode & 0x3f));
			}
			// surrogate pair
			else {
				i++;
				// UTF-16 encodes 0x10000-0x10FFFF by
				// subtracting 0x10000 and splitting the
				// 20 bits of 0x0-0xFFFFF into two halves
				charcode = 0x10000 + (((charcode & 0x3ff)<<10)
						| (str.charCodeAt(i) & 0x3ff));
				utf8.push(0xf0 | (charcode >>18), 
						0x80 | ((charcode>>12) & 0x3f), 
						0x80 | ((charcode>>6) & 0x3f), 
						0x80 | (charcode & 0x3f));
			}
		}
		return utf8;
	},
	
	Utf8ArrayToStr:function(array) {
		var out, i, len, c;
		var char2, char3;

		out = "";
		len = array.length;
		i = 0;
		while(i < len) {
			c = array[i++];
			switch(c >> 4)
			{ 
			case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7:
				// 0xxxxxxx
				out += String.fromCharCode(c);
				break;
			case 12: case 13:
				// 110x xxxx   10xx xxxx
				char2 = array[i++];
				out += String.fromCharCode(((c & 0x1F) << 6) | (char2 & 0x3F));
				break;
			case 14:
				// 1110 xxxx  10xx xxxx  10xx xxxx
				char2 = array[i++];
				char3 = array[i++];
				out += String.fromCharCode(((c & 0x0F) << 12) |
						((char2 & 0x3F) << 6) |
						((char3 & 0x3F) << 0));
				break;
			}
		}

		return out;
	},
	
	/**
	 * 把下标设置到起始位置
	 */
	Rewind:function( datas ){
		
		if ( datas != null )
			this._baseData = datas;
				
		this._bytesAvailable = this._baseData.length;
		this._pos = 0;
	},
	
	Clear:function(){
		this._baseData = [];
		this._pos = 0;
		this._bytesAvailable = 0;
	},
	
	_getSubArray:function (array, start, end) {
		if (array instanceof  Array)
			return array.slice(start, end);
		else
			return array.subarray(start, end);
	}
} );
/** 指示多字节数字的最高有效字节位于字节序列的最前面。*/
BinStream.BIG_ENDIAN = "bigEndian";

/** 指示多字节数字的最低有效字节位于字节序列的最前面。*/
BinStream.LITTLE_ENDIAN = "littleEndian";

BinStream.ZERO_BIT = "00000000000000000000000000000000";
BinStream.ONE_BIT = "11111111111111111111111111111111";
BinStream.RightMove = function( value, bitSize ){

};
BinStream.LeftMove = function( value, bitSize ){

};