﻿/**
 * 对象池
 * 使用前需手动调用Init设置 缓存最大个数 和 缓存对象类型
 * 缓存对象需要实现一个属性：bInPool；两个函数：Clean 和 Dispose 
 * pbk 2015年10月19日
 */
var MLObjectPool = cc.Class.extend({
	/**
	 * 初始化对象池
	 * @param maxCount 缓存最大个数
	 * @param cacheClass 缓存对象类型
	 */
	ctor:function( maxCount, cacheClass ){
		var self = this;
		
		self._cacheList = [];// 缓存对象列表
		self._nMaxCount = maxCount;   // 缓存最大个数
		self._cacheClass = cacheClass;// 缓存对象类型
	},
	
	/**
	 * 压入对象缓存
	 * @param cell
	 */
	Push:function( cell ){
		var self = this;
		
		if ( cell == null ){
			zz.Error( "ObjectPool cell is null!" );
			return;
		}

		if ( !(cell instanceof self._cacheClass) ){
			zz.Error( "ObjectPool cell isn't thie specify Class!" );
			return;
		}

		if ( cell.bInPool ){
			zz.Error( "ObjectPool cell in pool!" );
			return;
		}

		if ( self._cacheList.length < self._nMaxCount ){
			cell.Clean();
			cell.bInPool = true;
			self._cacheList.push( cell );
		} else {
			cell.Dispose();
		}
	},
	
	/**
	 * 弹出缓存对象
	 */
	Pop:function(){
		var self = this;
		
		if ( self._cacheList.length > 0 ){
			var cell = self._cacheList.pop();
			cell.bInPool = false;
			return cell;
		} else {
			return new self._cacheClass();
		}
	},
	
	/**
	 * 当前缓存池中对象个数
	 * @returns
	 */
	Length:function(){
		return this._cacheList.length;
	}
});