/**
 * 粒子效果
 */
var ParticleEffect = ParticleEffect || cc.Class.extend({
        /**
         * 粒子载体
         */
        container:null,
        /**
         * 粒子对象
         */
        particleList:null,
        /**
         * 粒子url
         */
        particleUrl:null,
        /**
         * 粒子生命周期
         */
        lifeCycle:null,
        /**
         *  粒子生成间隔时间
         */
        intervalTime:null,
        /**
         * 粒子散发半径
         */
        radiusMax:null,                          //最大半径
        radiusMin:null,                          //最小半径
        radiusRandomShow:null,                   //是否在范围内随机出现，如果为为falsh，就在最小半径出现，最大半径结束

        endValue:null,

        alphaTime2:null,                         //alpha从1到0开始渐变在周期内的时间点
        alphaTime1:null,                         //alpha从0渐变到1在周期内的时间点；

        angleRound:null,                        //发散角度
        angleRandom:null,                       //发散角度随机偏移值

        // 移动类型
        Normal:1,   // 匀速
        EaseIn:2,   // 由慢至快
        EaseOut:3,  // 由快至慢
        moveType:null,                          //粒子移动类型
        moveChangeTime:null,                    //粒子移动变速时间点比例 (相对于生命周期)

        RotateTimes:null,                          // 转动角度
        RotateAngle:null,                      // 时间



        timerUpdateId:null,                     //粒子生成计时器

        ctor:function(container, url, life, interval, radiusMin, radiusMax){
            var self = this;

            // init
            self.endValue = new Object();
            self.particleList = [];
            self.angleRound = 0;
            self.angleRandom = Math.PI;
            self.alphaTime1 = 0.1;
            self.alphaTime2 = 0.9;
            self.radiusRandomShow = false;
            self.RotateTimes = 0;
            self.RotateAngle = null;
            self.moveType = null;
            self.moveChangeTime = 0.3;

            self.timerUpdateId = 0;

            self.InitParticle(container, url, life, interval, radiusMin, radiusMax);
        },

        /**
         *
         * @param	particle_spr  粒子对象
         * @param	life_cycle 粒子生命周期
         * @param	interval_time 粒子生成间隔时间
         * @param	radius 粒子散发半径
         * @param	radius_min 粒子散发最小半径
         * @param	random_show 是否允许在设置半径内随机出现
         * @param	if_move 粒子是否从最小半径移动到最大半径
         */
        InitParticle:function (container, url, life, interval, radiusMin, radiusMax) {
            var self = this;

            self.container = container;
            self.particleUrl = url;
            self.lifeCycle = life;
            self.intervalTime = parseFloat(interval/1000);
            self.radiusMin = radiusMin;
            self.radiusMax = radiusMax;
        },

        Play:function () {
            if ( this.timerUpdateId == 0 )
                this.timerUpdateId = TimerUtil.TimeUpdate( this.CreateMovie, this, this.intervalTime );
        },

        CreateMovie:function () {
            var cell=this.particleList.pop();
            if( !cell ) {
                var cell = new cc.Sprite();
                var url = this.particleUrl;
                EngineMgr.uiMgr.GetAsynResByUrl(url, function(obj){
                    obj.initWithSpriteFrameName(url);
                },cell, cell);
                cell.alpha = 0;
                this.SetAlpha( cell );
                this.container.addChild(cell);
            }

            var angleNum = this.angleRound +(Math.random() * this.angleRandom * 2 - this.angleRandom);

            // 随机半径相关
            var randomLength = 0;
            if (this.radiusMin != 0)//如果设置了最小半径，半径就为最小半径
            {
                randomLength = this.radiusMin;
            }

            if ( this.radiusRandomShow )//如果允许在范围内随机出现，半径值为范围内随机值
            {
                randomLength = Math.random() * (this.radiusMax - randomLength) + randomLength;
            }

            cell.y = randomLength * Math.sin(angleNum);
            cell.x = randomLength * Math.cos(angleNum);

            if (this.moveType != null)
            {
                this.endValue.y = this.radiusMax * Math.sin(angleNum);
                this.endValue.x = this.radiusMax * Math.cos(angleNum);
            }

            this.Start(cell);
        },

        Start:function ( spr) {
            var life = this.lifeCycle;
            var action = cc.FadeIn( life*this.alphaTime1);
            var action1 = cc.FadeOut( life*this.alphaTime2 );
            var back = cc.CallFunc(function (cell,stage) {
                cell.stopAllActions();
                cell.x=0;
                cell.y=0;
                cell.alpha = 0;
                stage.SetAlpha( cell );
                stage.particleList.push(cell);
            },spr,this);
            spr.runAction(cc.Sequence(action, cc.delayTime((1-this.alphaTime1- this.alphaTime2)*life), action1, back));

            if (this.moveType != null ) {
                var times = life;
                var moveTo = new cc.MoveTo(times,cc.p(this.endValue.x,this.endValue.y));

                if (this.moveType == this.Normal)
                {

                }
                else if ( this.moveType == this.EaseIn ) {
                    moveTo = moveTo.easing(cc.easeIn(times*this.moveChangeTime));
                }
                else if ( this.moveType == this.EaseOut ) {
                    moveTo = moveTo.clone().easing(cc.easeOut(life*this.moveChangeTime));
                }

                spr.runAction( moveTo);
            }

            if (this.RotateAngle && this.RotateTimes)
            {
                var act1 = cc.RotateBy(this.RotateTimes, this.RotateAngle);
                this.container.runAction(cc.repeatForever(act1));
            }
        },

        SetAlpha:function(spr){
            if (spr.alpha < 0)
                spr.alpha = 0;
            if (spr.alpha > 1)
                spr.alpha = 1;
            spr.setOpacity(255 * spr.alpha);
        },

        /**
         * 设置粒子不允许随机方向发出
         * @param	angle 设置粒子发射角度，
         * @param   randomAngle 以angle为中心上下随机值
         */
        SetAngle:function (angle, randomAngle) {
            this.angleRound = angle * Math.PI / 180;
            this.angleRandom = randomAngle * Math.PI / 180;
        },

        // 设置旋转
        SetRotate:function (times, angle) {
            this.RotateTimes = times;
            this.RotateAngle = angle;
        },

        SetMove:function (type, timeRate) {
            this.moveType = type;
            this.moveChangeTime = timeRate;
        },

        SetRadiusRandomShow:function () {
            this.radiusRandomShow = true;
        },

        /**
         * 设置粒子显隐时间比例  (相对于生命周期)
         * @param	fadeIn 渐显，
         * @param   fadeOut 渐隐
         */
        SetFade:function (fadeIn, fadeOut) {
            this.alphaTime1 = fadeIn;
            this.alphaTime2 = fadeOut;
        },

        Dispose:function () {
            TimerUtil.ClearTimeUpdate(this.timerUpdateId, this);
            this.timerUpdateId = 0;
            this.container.stopAllActions();
            this.container.removeAllChildren();
            this.particleList.length = 0;
        }
});

ParticleEffect.Create = function ( container, url,life, interval, radiusMin, radiusMax ) {
    return new ParticleEffect( container, url,life, interval, radiusMin, radiusMax );
};

/***
 *  序列帧动画
 * @type {Function}
 */
 var AnimationEffect = AnimationEffect || cc.Class.extend({

        spr:null,
        plistUrl:null,
        frames:null,

        animalModel:null,

        intervalTime:null,
        ifRestorOriginalFrame:null,

        ctor:function(spr, num){
            var self = this;

            self.spr = spr;
            self.frames = num;

            self.animalModel = 0;
            self.intervalTime = 0.1;
            self.ifRestorOriginalFrame = false;
        },

        Play:function () {
            this.StartAni();
        },

        StartAni:function () {
            var imgUrl = this.spr.jsonobj.images[0];
            var pUrl = imgUrl.substring( 0 , imgUrl.lastIndexOf( "/"));
            var Suffix = imgUrl.substring(imgUrl.lastIndexOf( "."), imgUrl.length);

            var animation = new cc.Animation();
            for (var i = 1; i <= this.frames; i++) {
                var frameName = "res/ui/image/" + pUrl + "/" + i + Suffix;
                animation.addSpriteFrameWithFile(frameName);
            }

            animation.setDelayPerUnit(this.intervalTime);
            animation.setRestoreOriginalFrame(this.ifRestorOriginalFrame);

            var animal = cc.animate(animation);

            var pp = new cc.Sprite();
            pp.setAnchorPoint(0,0);
            pp.x = 0;
            pp.y = 0;
            this.spr.addChild(pp);
            if ( this.animalModel > 0 ) {
                var action = animal.repeat(this.animalModel)
                pp.runAction(action);
            }
            else {
                var action = cc.repeatForever( animal );
                pp.runAction(action);
            }
        },

        // 播放间隔
        SetInterval:function ( times ) {
            this.intervalTime = times;
        },

        // 播放完是否返回原始帧
        SetOriginalFrame:function () {
            this.ifRestorOriginalFrame = true;
        },

        // 播放次数或无限循环   ( 0:循环， >0： 次数 )
        SetAnimalModel:function ( model ) {
            if ( model == undefined || model <= 0 )
                this.animalModel = 0;
            else
                this.animalModel = model;
        }
 });

AnimationEffect.Create = function ( spr, num ) {
    return new AnimationEffect( spr, num );
};