﻿/** 场景背景音乐*/
var SoundType = SoundType || {};
SoundType.BGM = 100;
/** UI音频*/
SoundType.UI = 101;
/** 动作音频*/
SoundType.Action = 102;
/** 技能音频*/
SoundType.Skill = 103;
/** 场景音频*/
SoundType.Scene = 104;
/** 脚步音频*/
SoundType.Foot = 105;
/** 字幕说话音频*/
SoundType.BossTalk = 106;
/** NPC说话音乐*/
SoundType.NpcTalk = 107;

/**
 * 游戏音频管理
 * 效果1：根据攻击动作的时间，动态调整音频播放长度（加进和慢放？）
 * 效果2：材质匹配声音
 * 效果3：随机声音选择
 * 效果4：攻击音效，挥刀声音+击中声音
 * pbk 2015年10月27日
 */
var GameSoundMgr = SoundMgr.extend({
	musicArr:null,
	soundHash:null,

    bgmId:null,
	sceneTimeId:-1,
	ctor:function(){
		var self = this;
		
		self._super();

		self._sPreUrl = "res/sound/";

		self._sceneSoundList = [];// 场景音频列表
		
		self._InitConfig();

		return self;
	},
	
	/**
	 * 播放音频
	 * @param name 音频文件名
	 * @param type 音频类型
	 * @param loop 是否循环播放
	 * @param args 额外属性
	 * @return 音频ID，之后很多操作都需要ID来进行
	 */
	PlaySound:function( name, type, loop, endCB ){
		if(this._bMute){
			return null;
		}
		loop = loop || false;
		endCB = endCB || null;
		var self = this;
		var url = self._sPreUrl;
		if ( type === SoundType.BGM ){// 背景音乐
			url += "map/";
		} else if ( type === SoundType.UI ){// UI音频
			url += "ui/";
		} else if ( type === SoundType.Action ){// 动作音频
			url += "action/";
		} else if ( type === SoundType.Skill ){// 技能音频
			url += "skill/";
		} else if ( type === SoundType.Scene ){// 效果音频
			url += "scene/";
		} else if ( type === SoundType.Foot ) {// 脚步音频
			url += "foot/";
		} else if ( type === SoundType.BossTalk ){// Boss说话音频
			url += "talk/boss/boss_";
		} else if ( type === SoundType.NpcTalk){// NPC说话声
			url += "talk/npc/npc_";
		}

		url += name + ".mp3";
		if( type === SoundType.BGM )
			self._mapSoundUrl = url;
		else if( !this._isPlaySound )
			return null;

		var param = {name:name, type:type, loop:loop, id:self._toId++, endCB:endCB};
		LoadMgr.LoadSound( url, self, LoadPriority_Preload,
				{complete:self._LoadSoundComplete, completeParam:param} );

		return param.id;
	},

	PlaySoundSkill:function(skillId){
		var soundId = "1_1";
		switch( skillId ){
			case 10041:
				soundId = "6";
				break;
			case 10042:
				soundId = "7";
				break;
			default:
				var id = skillId % 10;
				if( id > 5 )
					id = 5;
				if( id == 1 )
					soundId = id + "_" + ToolBase.RamdonRangeInt(1, 3);
				else
					soundId = id + "_" + ToolBase.RamdonRangeInt(1, 4);
				break;
		}
		this.PlaySound(soundId,SoundType.Skill);
	},

	PlayBGM:function(){
		TimerUtil.ClearTimeout(this.musicTimeId);
		this.musicTimeId = -1;
		if( this.musicArr ){
			this.curMusic = this.musicArr[ToolBase.RamdonRangeInt(0, this.musicArr.length)];
			this.PlaySound( this.curMusic.id, SoundType.BGM, false, {cb:this.PlayBGMFinish, target:this, args:null} );
		}
	},

	PlayBGMFinish:function(){
		if( this.curMusic ){
			this.musicTimeId = TimerUtil.SetTimeout( this, this.PlayBGM, this.curMusic.delay * 1000 );
		}
		else{
			this.PlayBGM();
		}
	},

	PlaySoundScene:function(){
		TimerUtil.ClearTimeout(this.sceneTimeId);
		this.PlaySound( ToolBase.RamdonRangeInt(1, 4), SoundType.Scene, false );
		this.sceneTimeId = TimerUtil.SetTimeout( this, this.PlaySoundScene, ToolBase.RamdonRangeInt(25, 60) * 1000 );
	},

	PlaySoundAction:function(action){
		if( action == EnumActionType.RUN ){
			if( !this.walkTimeId ){
				this.PlaySoundFoot();
				this.walkTimeId = TimerUtil.TimeUpdate( this.PlaySoundFoot, this, 0.36 );
			}
		}
		else{
			if( this.walkTimeId ){
				TimerUtil.ClearTimeUpdate( this.walkTimeId, this );
				this.walkTimeId = 0;
			}
		}
		switch (action){
			case EnumActionType.JUMP:
			case EnumActionType.JUMP_FLY:
			case EnumActionType.JUMP_SKY2:
				this.PlaySound( "jump_" + ToolBase.RamdonRangeInt(1, 6), SoundType.Action );
				break;
			case EnumActionType.ATTACK:
			case EnumActionType.ATTACK2:
			case EnumActionType.ATTACK3:
			case EnumActionType.ATTACK4:
			case EnumActionType.ATTACK5:
			case EnumActionType.ATTACK6:
			case EnumActionType.ATTACK7:
				this.PlaySound( (Mgr.mainPlayer.nJobId == 1 ? "male_" : "female_") + ToolBase.RamdonRangeInt(1, 7), SoundType.Action );
				break;
		}
	},

	PlaySoundFoot:function(){
		// if ( args.isRide ){
		// 	if ( args.horseId <= 202 ){
		// 		url += "horse_";
		// 	} else {
		// 		url += "other_";
		// 	}
		// } else {
		if( !Mgr.mainPlayer ){
			if( this.walkTimeId ){
				TimerUtil.ClearTimeUpdate( this.walkTimeId, this );
				this.walkTimeId = 0;
			}
			return;
		}
		var gridPt = Mgr.mainPlayer.GetGridPt();
		var type;
		if( Mgr.sceneMap.IsInWater(gridPt.x, gridPt.y) || Mgr.sceneMap.IsInWaterless(gridPt.x, gridPt.y) ){
			type = "inless";
		}
		else{
			type = "land";
		}
		var id = this.GetSoundIdByType(type);
		if( id ){
			this.PlaySound(type + "_" + id, SoundType.Foot);
		}
		// }

	},

	PlaySoundHit:function(){
		return;	//好像没啥感觉 跟技能的重再一起听不到了
		var id = this.GetSoundIdByType("hit");
		if( id ){
			this.PlaySound("hit_" + id, SoundType.Action);
		}
	},

	GetSoundIdByType:function(type){
		var randomArr = this.soundHash.Get(type);
		var random = Math.random();
		if( randomArr ){
			for( var i = 0, len = randomArr.length; i < len; i++ ){
				var obj = randomArr[i];
				if( random <= obj.chance ){
					return obj.list[ToolBase.RamdonRangeInt(0, obj.list.length)];
				}
			}
		}
		return 0;
	},
	
	// 获取音频完成
	_LoadSoundComplete:function( audio, args ){
		var self = this;
		if ( DelArrayElement( self._operateClean, args.id ) ){
			return;
		}
		audio.id = args.id;
		// 能否播放
		var canPlay = true;//!HasArrayElement( self._operateStop, args.id ) && !self._bMute;
		if ( args.type === SoundType.BGM ){// 背景音频
			if ( audio.url != self._mapSoundUrl )
				return;
			self._AddMusic( audio );
			if ( canPlay ){
				audio.Play( audio, args.loop, args.endCB );
			}
		}
		else
		{
			if ( canPlay ){
				self._AddSound( audio );
				audio.Play( audio, args.loop, args.endCB );
			}
		}
		/*
		if ( args.type === SoundType_Round ){// 范围音频
			if ( args.sound.bDispose )
				return;

			self._AddSound( audio );
			args.sound.SetAudio( audio );

			if ( canPlay && self._bindFoucs ){
				args.sound.UpdateVolume( self._bindFoucs );
			}
		} else if ( args.type === SoundType_Trace ){// 轨迹音频
			if ( args.sound.bDispose )
				return;
			
			self._AddSound( audio );
			args.sound.SetAudio( audio );
			
			if ( canPlay && self._bindFoucs ){
				args.sound.UpdateVolume( self._bindFoucs );
			}
		} else if ( args.type === SoundType_BGM ){// 背景音频
			if ( args.name != self._mapSoundId )
				return;
			self._AddSound( audio );
			if ( canPlay ){
				audio.Play( audio, args.loop, false );
			}
		}
		else
		{
			if ( canPlay ){
				self._AddSound( audio );
				audio.Play( audio, args.loop, true );
			}
		}
		 else if ( args.type === SoundType_UI ){// UI音频
			if ( canPlay ){
				self._AddSound( audio );
				audio.Play( audio, args.loop, true );
			}
		} else if ( args.type === SoundType_Action ){// 动作音频
			if ( canPlay ){
				self._AddSound( audio );
				audio.Play( audio, args.loop, true );
			}
		} else if ( args.type === SoundType_Skill ){// 技能音频
			if ( args.loop ){
				self._AddSound( audio );

				if ( canPlay ){
					audio.Play( 0, args.loop );
				}
			} else {
				if ( canPlay ){
					self._AddSound( audio );

					audio.Play( 0, args.loop, {cb:self._DelSound, target:self, args:audio} );
				}
			}
		} else if ( args.type === SoundType_Ghost ){// 效果音频
			if ( args.loop ){
				self._AddSound( audio );

				if ( canPlay ){
					audio.Play( 0, args.loop );
				}
			} else {
				if ( canPlay ){
					self._AddSound( audio );

					audio.Play( 0, args.loop, {cb:self._DelSound, target:self, args:audio} );
				}
			}
		} else if ( args.type === SoundType_Foot ){// 脚步音频
			if ( args.loop ){
				self._AddSound( audio );

				if ( canPlay ){
					audio.Play( 0, args.loop );
				}
			} else {
				if ( canPlay ){
					self._AddSound( audio );

					audio.Play( 0, args.loop, {cb:self._DelSound, target:self, args:audio} );
				}
			}
		} else if ( args.type === SoundType_Talk ){// 字幕说话音频
			if ( args.loop ){
				self._AddSound( audio );

				if ( canPlay ){
					audio.Play( 0, args.loop );
				}
			} else {
				if ( canPlay ){
					self._AddSound( audio );

					audio.Play( 0, args.loop, {cb:self._DelSound, target:self, args:audio} );
				}
			}
		} else if ( args.type === SoundType_NPCTalk){// NPC说话声
			if ( args.loop ){
				self._AddSound( audio );

				if ( canPlay ){
					audio.Play( 0, args.loop );
				}
			} else {
				if ( canPlay ){
					self._AddSound( audio );

					audio.Play( 0, args.loop, {cb:self._DelSound, target:self, args:audio} );
				}
			}
		} else if ( args.type === SoundType_Com ){// 公共音频
			if ( args.loop ){
				self._AddSound( audio );

				if ( canPlay ){
					audio.Play( 0, args.loop );
				}
			} else {
				if ( canPlay ){
					self._AddSound( audio );

					audio.Play( 0, args.loop, {cb:self._DelSound, target:self, args:audio} );
				}
			}
		}
		 */
	},
	
	/**
	 * 清理音频
	 * @param id
	 */
	CleanSound:function( id ){
		if ( this._DelSceneSoundByID( id ) || this._DelDynaSoundByID( id ) ){
			return;// 如果是场景音频或者范围音频，并操作成功，则跳出
		}

		var audio = this._soundIdMap[id];
		if ( audio ){
			this._DelSound( audio );
		} else {
			this._operateClean.push( id );
		}
	},
	
	/**
	 * 清理所有音频
	 */
	CleanAllSound:function(){
		var self = this;

		self.CleanAllSceneSound();
		self._DelAllDynaSound();

		self._DelAllSound();

		zz.Log( "清理所有音频！！！" );
	},
	
	// ====================================================================================================
	// ====================================================================================== 场景音频
	// ====================================================================================================
	/**
	 * 设置场景音频
	 */
	SetSceneSound:function( datas ){
		var self = this;

		self.CleanAllSceneSound();

		if ( !datas )
			return;

		try{
			var stream = new BinStream( datas );
			stream.endian = BinStream.LITTLE_ENDIAN;

			var sound, name, id, r, x, y;

			while ( stream.bytesAvailable() > 0 ){
				name = stream.ReadString();
				id = stream.ReadInt();
				r = stream.ReadInt() * MGameDefine.GRID_WIDTH;
				x = stream.ReadInt();
				y = stream.ReadInt();

				zz.Log( "创建场景音频：" + name + " " + id + " " + x + " " + y + " " + r );
				sound = new RoundSound( true, x, y, r );
				LoadMgr.LoadSound( self._sPreUrl + "scene/" + id + ".mp3", this, LoadPriority_Preload, 
						{complete:self._LoadSoundComplete, completeParam:{type:SoundType_Round, sound:sound}} );

				if ( AddArrayElement( self._sceneSoundList, sound ) ){
					self._AddDynaSound( sound );
				}
			}
		} catch( e ){
			zz.Error( "GameSoundMgr：" + e );
		}
	},
	
	// 根据ID删除场景音乐
	_DelSceneSoundByID:function( id ){
		var list = this._sceneSoundList;
		var len = list.length;

		for ( var i = 0; i < len; ++i ){
			if ( list[i].GetID() === id ){
				list.splice( i, 1 );
				this._DelDynaSound( list[i] );
				return true;
			}
		}

		return false;
	},
	
	/**
	 * 清理所有场景音频
	 */
	CleanAllSceneSound:function(){
		var self = this;

		self._sceneSoundList.forEach( function( rs ){
			self._DelDynaSound( rs );
		} );

		self._sceneSoundList.length = 0;
	},
	
	// ====================================================================================================
	// ====================================================================================== 材质匹配+随机
	// ====================================================================================================
	/**
	 * 初始化配置
	 * 配置格式：类型，材料类型1，材料类型2，urls(url;url;...)
	 * 
	 */
	_InitConfig:function(){
		this._matchDic = {};// 材质匹配字典
		
		
	},
	/**
	 * 暂停所有音频
	 */
	PauseSound:function( id ){
		var audio = new AudioEx();
		audio.Pause();
	},

	/**
	 * 恢复所有音频
	 */
	ResumeSound:function( id ){
		var audio = new AudioEx();
		audio.Resume();
	}
});