var TweenParticleMovie = ccui.Widget.extend(/** @lends cc.Node# */{
	/**
	 * 粒子对象
	 */
	particleSpr:null,
	/**
	 * 粒子生命周期
	 */
	lifeNum:0,
	/**
	 *  粒子生成间隔时间
	 */
	intervalTime:0,
	/**
	 * 粒子散发半径
	 */
	radiusNum:0,//最大半径
	radiusMin:0,//最小半径
	radiusRandomShow:false,//是否在范围内随机出现，如果为为falsh，就在最小半径出现，最大半径结束
	ifMove:false,//是否移动

	tweenTimer:-1,
	endValue:null,
	roundTweenMc:null,



	angleRound: 0,//角度
	angleRandom:Math.PI,//角度随机偏移值

	alphaTime2:0,//alpha从1到0开始渐变在周期内的时间点
	alphaTime1:0.2,//alpha从0渐变到1在周期内的时间点；

	ifRound:true,//粒子是否自转
    isStart:false,
	/**
	 *
	 * @param	particle_spr  粒子对象
	 * @param	life_num 粒子生命周期
	 * @param	interval_time 粒子生成间隔时间
	 * @param	radius_num 粒子散发半径
	 * @param	radius_min 粒子散发最小半径
	 * @param	random_show 是否允许在设置半径内随机出现
	 * @param	if_move 粒子是否从最小半径移动到最大半径
	 */
	ctor: function(particle_spr, life_num, interval_time, radius_num, radius_min, random_show, if_move){
		this._super();

		if(life_num == null)
			life_num = 3;
		if(interval_time == null)
			interval_time = 0.2;
		if(radius_num == null)
			radius_num = 80;
		if(radius_min == null)
			radius_min = 0;
		if(random_show == null)
			random_show = false;
		if(if_move == null)
			if_move = true;
		this.endValue = {};
		this.ifMove = if_move;
		this.particleSpr = particle_spr;
		this.lifeNum = life_num;
		this.intervalTime = interval_time;
		this.radiusNum = radius_num;
		this.radiusRandomShow = random_show
		this.radiusMin = radius_min;
	},

	/**
	 * 播放粒子动画
	 */
	 play:function ()
	 {
         this.isStart = true;
		 TimerUtil.ClearTimeout(this.tweenTimer);
		 this.tweenTimer = TimerUtil.TimeUpdate(this.tweenTimerEvent, this, this.intervalTime);
	 },
	 /**
	  * 设置粒子不允许随机方向发出
	  * @param	num 设置粒子发射角度，
	  * @param   _randomangle 以num为中心上下随机值
	  */
	 setAngle:function(num, _randomangle)
	 {
		 if(_randomangle == null)
			 _randomangle = 0;
		 this.angleRound = num * Math.PI / 180;
		 this.angleRandom = _randomangle * Math.PI / 180;
	 },
	 /**
	  * 设置旋转
	  */
	 roundTween:function(speed)
	 {
		 var rotateby = new cc.RotateBy(10, 360).repeatForever();
		 this.runAction(rotateby);
	 },
	 /**
	  * 设置alpha在周期内显示和消失关键帧的位置
	  * @param	_alphatime1 alpha从0到1，其中1的关键帧位置
	  * @param	_alphatime2 alpha从1到0，其中1的关键帧位置
	  */
	 setAlpha:function(_alphatime1,_alphatime2)
	 {
		 if(_alphatime1 == null)
			 _alphatime1 = 0.2;
		 if(_alphatime2 == null)
			 _alphatime2 = 0;
		 this.alphaTime2 = _alphatime2;
		 this.alphaTime1 = _alphatime1;

	 },
	 /**
	  * 设置粒子是否自转
	  * @param	_ifround
	  */
	 setRound:function setRound(_ifround)
	 {
		 if(_ifround == null)
			 _ifround = true;
		 this.ifRound = _ifround
	 },
	 tweenTimerEvent:function()
	 {
         if(!this.isStart){return;}
		 var particle = new MUIImage();
		 particle.ParseJson( this.particleSpr.jsonobj );
		 particle.Init();
		 particle.x = -(particle.width / 2);
		 particle.y = -(particle.height / 2);
		 var spr = new ccui.Widget();
		 spr.addChild(particle);
		 spr.alpha = 0;
		 this.addChild(spr);
		 var angleNum = 0;
		 angleNum = this.angleRound +(Math.random() * this.angleRandom * 2 - this.angleRandom);

//		 ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓半径相关↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
		 var randomLendht = 0;
		 if (this.radiusMin != 0)//如果设置了最小半径，半径就为最小半径
		 {
			 randomLendht = this.radiusMin;
		 }

		 if (this.radiusRandomShow)//如果允许在范围内随机出现，半径值为范围内随机值
		 {
			 randomLendht = Math.random() * (this.radiusNum - randomLendht) + randomLendht;
		 }
//		 ↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑↑

		 spr.y = randomLendht * Math.sin(angleNum);
		 spr.x = randomLendht * Math.cos(angleNum);

		 if (this.ifMove)
		 {
			 this.endValue.y = this.radiusNum * Math.sin(angleNum);
			 this.endValue.x = this.radiusNum * Math.cos(angleNum);
		 }
		 /*endValue.onComplete = lineComp;
 endValue.onCompleteParams = [spr,particle];*/

		 var tween = new cc.Spawn(new cc.MoveTo(this.lifeNum, cc.p( this.endValue.x, this.endValue.y )))
		 spr.runAction( cc.Sequence( cc.DelayTime( 0 ), tween ) );

		 if (this.alphaTime2 == 0)
		 {
			 var tween = new cc.Spawn( new cc.FadeOut(this.lifeNum * (1-this.alphaTime1)));
			 particle.runAction( cc.Sequence( cc.DelayTime( this.lifeNum*this.alphaTime1 ), tween, cc.callFunc(this.lineComp, spr) ) );
		 }else{
			 var tween = new cc.Spawn( new cc.FadeOut(this.lifeNum * (1-this.alphaTime2)));
			 particle.runAction( cc.Sequence( cc.DelayTime( this.lifeNum*this.alphaTime2 ), tween, cc.callFunc(this.lineComp, spr) ) );
		 }
		 
		 if (this.ifRound)
		 {
			 var rotateby = new cc.RotateBy(this.lifeNum, 360).repeatForever();
			 spr.runAction(rotateby);
		 }

		 /*			spr.removeChild(particle);
 particle.bitmapData.dispose();*/

	 },

	 cloneObj:function(obj)
	 {
		 var newObj = new Object();
		 for (var names in obj)
		 {
			 newObj[names] = obj[names]
		 }
		 return newObj
	 },

	 lineComp:function(spr)
	 {
		 spr.stopAllActions();
		 UIUtil.UIRemoveSelf(spr);
		 spr = null;
	 },

	 removeThisAll:function(e)
	 {
         this.isStart = false;
		 TimerUtil.ClearTimeout(this.tweenTimer);
		 this.tweenTimer = -1;
		 this.stopAllActions();
	 }
});